<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

/**
 *	This is the main configuration file of the Woogle extension.
 *
 *	It contains a number of configuration settings in the config array and several utility
 * 	methods to access these settings and more.
 */
class WoogleConfig {
	
	/* configure Woogle here
		all these parameters can be read using WoogleConfig::get('key')
		
	    These are DEFAULT values - do not change. To change values call
	    WoogleConfig::set('key', 'value') in LocalSettings.php
	*/
	private static $config = array(
		// key                      // value                   // comment
		
		// core features
		'indexAddon'                => array('Native',         'addon used for the Woogle index (either "Native" or "Remote") - will be automatically overwritten by actual addon if empty'),
		'indexFileContent'          => array(false,            'add content of uploaded files to index'),
		'htmlIndexing'              => array(false,            'write parsed articles to the index instead of plain wiki-text'),
		'core'                      => array(true,             'enable or disable woogle at all'),
		'replace'                   => array(true,             'replace built-in MediaWiki search'),
		'searchWikiOnlyAsDefault'   => array(true,             'if multiple repositories are present, search only wiki itself by default'),
		'advanced'                  => array(true,             'use Woogle "advanced" mode: redirection to an own wiki page for each search result'),
		'advancedBox'               => array(true,             'show the box for the "advanced" mode with the corresponding wiki content in it'),
		'advancedWikiLink'          => array(true,             'show a button to create a wiki page with the same name as the search string'),
		'advancedWatch'             => array(true,             'show a button to watch a search result for changes in the corresponding wiki page'),
	    'advancedDiscussion'       	=> array(true,             'show a button including a link on the result\'s discussion page'),
		'advancedIcons'             => array(true,             'show metadata icons'),
		'advancedNamespaceGrab'     => array(true,             'if the (usually) Woogle:Namespace should be grabbed and redirected to the nonAdvanced url if advanced is turned off'),
		'redLinkInfo'               => array(true,             'show meta-info popups next to red links'),
		'limitAccessToGroups'       => array(false,            'access limited to mediawiki groups specified in $groups'),
		'accessGroups'              => array(array('sysop'),   'groups access is limited to if $limitAccessToGroups'),
		'useWoogleDefault'          => array(1,                'default user setting if Woogle is enabled after its deployment - 0 or 1 - 0 means users have to manually enable Woogle in their preferences'),
		'watchSearchResults'		=> array(true,			   'if set, watches on woogle pages will inform users about new/updated search results ("WoogleAlerts")'),
		'namespaceIndex'			=> array(150,			   'namespace index for WOO_NS_MAIN and following. Note: this value should not be changed using the set() method'),
		'localWikiPath'				=> array('',	  		   'default $wgServer - if set, all extensions of this path will be shown as local results in search result lists'),
		'excludeNamespaces'			=> array(array(NS_MEDIAWIKI),		'namespaces that should be exluded from indexing by Woogle'),
		'stripMWMarkup'				=> array(array(), 				'consists of two-element arrays containing regular expression and its replacement for stripping MediaWiki markup before indexing'),
	
		// Ajax-Features
		'socialResults'             => array(false,            'offers the possibility to add, remove and rank search results'),                
		'autoCompleteDebug'         => array(false,            'autoCompleteDebug-notice, for testing if autoComplete works (creating the box), even if there are no Search-Index'),
		'autoComplete'              => array(true,             'autoComplete activation for WoogleSearch and MediaWiki build in search'),
		'restrictAutoComplete'      => array(false,            'limits the autocomplete function to suggest only existing document titles. useful for large wikis.'),
		'inPlaceEditing'            => array(true,             'edit information need description on woogle pages inline using Ajax instead of making http request to bring up edit form'),
	
		// result presentation UI settings
		'defaultCount'              => array(10,               'default number of results shown per page'),
		//'helpPage'                  => array('Woogle',         'wiki-page shown as help page'),
		'resultWidth'               => array(650,              'pixel-width of the result list'),
		'clickTracking'             => array(true,             'if clicks are being tracked'),
		'bigImagesOnHover'			=> array(true,			   'shows bigger images in results on Mouseover'),
		'editInternalResult'		=> array(true,		       'shows edit link for internal wiki page search results'),
	
		// technical settings
		'useProfiling'              => array(false,            'if profiling is used'),
		'useLogging'                => array(true,             'if log files are written'),
		'writeLogsToShell'      	=> array(false,            'if log output is written to system out instead of files (typically in testing mode)'),
		'usePerformanceLogging'     => array(true,             'if a performance log file is written'),
		'useDebugLogging'           => array(false,            'if debug log files are written (will produce lots of output)'),
		'useEMailLogging'           => array(false,            'if log messages (errors) should be send by mail'),
		'adminEMail'                => array('',               'E-Mail adress for admin notifications - if empty, the MediaWiki default is used'),
		'logQuery'					=> array(false,			   'if a text-based query log should be maintained')
	
	);
	// *** end of configuration ***
	
	private static $configLoaded = false;

	private static $preferences;

	
	//
	// Configuration access methods
	//
	
	
	/**
	 * Retrieve configuration values from $config array
	 * 
	 * @param 		$key name of configuration parameter
	 * @param		$loadconf  if lconfig hook should be called (should be avoided while processing LocalSettings.php
	 * @return 		value of configuration parameter
	 */
	public static function get($key, $loadconf=true) {
		WoogleUtil::profileIn(__METHOD__);
		if ($loadconf) self::loadConfig();
		$val = self::$config[$key][0];
		if (!isset($val)) WoogleUtil::debug('Unknown configuration parameter ' . $key);
		WoogleUtil::profileOut(__METHOD__);
		return $val;
	}
	
	/**
	 * Retrieve configuration value descriptions from $config array
	 * 
	 * @param 		$key name of configuration parameter
	 * @return 		description/comment of configuration parameter
	 */
	public static function getComment($key) {
		WoogleUtil::profileIn(__METHOD__);
		self::loadConfig();
		$ret = self::$config[$key][1];
		if (!isset($ret)) $ret = '(No comment defined for configuration parameter ' . $key .')'; 
		WoogleUtil::profileOut(__METHOD__);
		return $ret;
	}
	
	/**
	 * Set configuration parameter (to be used in LocalSettings.php)
	 * 
	 * E.g. to change 
	 * 		$config['remoteAuthKey']			= array("anonymous",	'authKey being passed to the remote backend');
	 *
	 * add to LocalSettings.php (*after* including Woogle.php)
	 * 		WoogleConfig::set('remoteAuthKey', '123');
	 * 
	 * @param		$key	name of configuration parameter
	 * @param		$value	configuration parameter value
	 * @return		void
	 */
	public static function set($key, $value){
		WoogleUtil::profileIn(__METHOD__);
	
		if (isset(self::$config[$key][0])){
			self::$config[$key][0] = $value;
		} else {
			WoogleUtil::error('Can not set unknown configuration parameter ' . $key);
		}
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * Set configuration parameter to array (used by addons)
	 * 
	 * @param		$key	name of configuration parameter
	 * @param		$value	configuration parameter array (containing value and comment)
	 * @return		void
	 */
	public static function setNewValue($key, $arr){
		WoogleUtil::profileIn(__METHOD__);
		self::$config[$key] = $arr;
		WoogleUtil::profileOut(__METHOD__);
	}
	
	
	/**
	 * Retrieve all configuration parameters used in the $config array
	 * 
	 * @return 		array containing configuration parameters
	 */
	public static function getKeys() {
		WoogleUtil::profileIn(__METHOD__);
		self::loadConfig();
		$keys = array_keys(self::$config);
		WoogleUtil::profileOut(__METHOD__);
		return $keys;
	}
	
	
	/**
	 * Pulls configuration values from addons by running the WoogleConfig hook
	 * 
	 * @return 	void
	 */
	private static function loadConfig() {
		WoogleUtil::profileIn(__METHOD__);
		if (!self::$configLoaded) {
			// set $configLoaded before running hooks, in case the hook wants to access get() resulting in an infinite recursion otherwise
			self::$configLoaded = true;
			wfRunHooks('WoogleConfig', array(&self::$config));
		}
		WoogleUtil::profileOut(__METHOD__);
	}
		
	//
	//	Utility methods for user preferences
	//	
	
	public static function getPreferences() {
		global $wgUser;
		WoogleUtil::profileIn(__METHOD__);
		// load config first if preferences are needed by config hooks
		if (!self::$configLoaded) {
			self::loadConfig();
		}
		
		//
		//	TAKE CARE! Full option key ('WoogleGeneralUseWoogle') may not exceed 32 chars!
		//
		
		//
		//	Initialize Woogle core user settings
		//
		// see \languages\WoogleLang*.php for description/localization of settings 
		if (empty(self::$preferences)) {
			self::$preferences = array(
				'General' => array(
					'UseWoogle' => array(
						'type' => 'checkbox',		// TODO make constants in WoogleMW
						'default' => WoogleConfig::get('useWoogleDefault'),
					),
					'ReplaceMWS' => array(
						'type' => 'checkbox',
						'default' => 1,
					),
				),
				/*'Sharing' => array(
					'Contribution' => array(
						'type' => 'checkbox',
						'default' => 1,
					),
				),*/
				'Privacy' => array(
					'TrackUserQueries' => array(
						'type' => 'checkbox',
						'default' => 1,
					),
					'RandomUserID' => array(
						'type' => 'text-nonempty',
						'initval' => md5(rand()),
						'dynamic' => true,
					),
					//'QueryStopwords' => array(
					//	'type' => 'text',
					//),
				),
			);
			// run hook to add preferences as e.g. defined by addons
			wfRunHooks('WooglePreferences', array(&self::$preferences));

			
			//
			// Store dynamic values if set initially
			//
			if (!$wgUser->isAnon()) {
			
				$saveSettings = false;
				// iterate all groups
				foreach (self::$preferences as $groupKey => $groupValue) {
					// iterate all items
					foreach ($groupValue as $prefKey => $prefValue) {
					
						// select dynamic values only
						$name = self::getPreferenceName($groupKey, $prefKey);
						if (array_key_exists('dynamic', $prefValue)) {
							// if value was not set before, it will be null
							if ($wgUser->getOption($name) == null){
							
								WoogleUtil::info('INST1 ' . $wgUser->getOption($name));
								WoogleUtil::info('INST2 ' . $wgUser->getOption('WoogleInstInstGroup'));
								WoogleUtil::info('INST3 ' . $wgUser->getOption('WooglePrivacyRandomUserID'));
							
								// persist
								$wgUser->setOption($name, $prefValue['initval']);
								WoogleUtil::info('Setting initval for ' . $name . ' to ' . $prefValue['initval']);
								$saveSettings = true;
							}
							self::$preferences[$groupKey][$prefKey]['default'] = $prefValue['initval'];	// set default only after saving
						}
					}
				}
				if ($saveSettings) {
					$wgUser->saveSettings();  // This will fire http://www.mediawiki.org/wiki/Manual:Hooks/UserSaveOptions
				}

			}

		}
		WoogleUtil::profileOut(__METHOD__);
		return self::$preferences;
	}

	
	public static function getPreferenceName($groupKey, $preferenceKey) {
		WoogleUtil::profileIn(__METHOD__);
		$prefname = 'Woogle' . $groupKey . $preferenceKey; // keep string short (32 chars)
		WoogleUtil::debug('Prefname length: ' . strlen($prefname) . ' ' . $prefname);
		return WoogleUtil::profileOut(__METHOD__, $prefname);	
	}
	
	public static function getPreferenceType($groupKey, $preferenceKey) {
		WoogleUtil::profileIn(__METHOD__);
		$prefs = self::getPreferences();
		$type = 'text';
		if (array_key_exists('type', $prefs[$groupKey][$preferenceKey])) {
			$type = $prefs[$groupKey][$preferenceKey]['type'];
		} 
		WoogleUtil::profileOut(__METHOD__);
		return $type;
	}
	
	public static function getPreferenceDefault($groupKey, $preferenceKey) {
		WoogleUtil::profileIn(__METHOD__);
		$prefs = self::getPreferences();
		//$default = '';
		if (array_key_exists('default', $prefs[$groupKey][$preferenceKey])) {
			$default = $prefs[$groupKey][$preferenceKey]['default'];
		} 
		WoogleUtil::profileOut(__METHOD__);
		return $default;
	}
	
	public static function getPref($groupKey, $preferenceKey) {
		global $wgUser;
		WoogleUtil::profileIn(__METHOD__);
		$name 	= self::getPreferenceName($groupKey, $preferenceKey);
		$prefs 	= self::getPreferences();
		$pref 	= $prefs[$groupKey][$preferenceKey];
		if ($wgUser->isAnon()) {
			if (array_key_exists('anon', $pref)) {
				return WoogleUtil::profileOut(__METHOD__, $pref['anon']);
			} elseif (array_key_exists('default', $pref)) {
				return WoogleUtil::profileOut(__METHOD__, $pref['default']);
			} else {
				WoogleUtil::error('Undefined preference value ' . $groupKey . ' / ' . $preferenceKey . '  for anon user');
				return WoogleUtil::profileOut(__METHOD__, '');
			}
		} else {
			return WoogleUtil::profileOut(__METHOD__, $wgUser->getOption($name, $pref['default']));
		}
	}
	
	public static function setPref($groupKey, $preferenceKey, $value) {
		global $wgUser;
		WoogleUtil::profileIn(__METHOD__);
		$name = self::getPreferenceName($groupKey, $preferenceKey);
		
		// only save for registered users
		if (!$wgUser->isAnon()) {
			$wgUser->setOption($name, $value);
			$wgUser->saveSettings();
		}
		WoogleUtil::profileOut(__METHOD__);
	}
	
	
	//
	// Woogle config utility methods
	//
	
	/**
	 * Tells if Woogle is globally enabled AND activated by the user
	 * @return boolean
	 */
	public static function isEnabled(){
		return (WoogleConfig::get('core') && WoogleConfig::getPref('General', 'UseWoogle'));	
	}
	
	/**
	 * Tells if Woogle is configured to globally replace MW Search AND by the user
	 * @return boolean
	 */
	public static function isReplacingMWS(){
		return (WoogleConfig::get('replace') && WoogleConfig::getPref('General', 'ReplaceMWS'));
	}
	
	/**
	 * Checks if the current user is permitted to use Woogle
	 * @return 	boolean, yes/no
	 */
	public static function hasPermission() {
		WoogleUtil::profileIn(__METHOD__);
		if (!self::get('limitAccessToGroups')) {
			return true;
		}
		$group = self::getAccessGroup();
		WoogleUtil::profileOut(__METHOD__);
		return !empty($group);
	}
	
	/**
	 * Returns one group defined in $config['accessGroups'] which overlaps with the
	 * groups in which the current user is member of 
	 * 
	 * @return String, empty or name of one group
	 */
	public static function getAccessGroup() {
		global $wgUser;
		WoogleUtil::profileIn(__METHOD__);
		// block if user not in one of $groups
		$userGroups = $wgUser->getGroups();
		foreach (self::get('accessGroups') as $group) {
			foreach ($userGroups as $userGroup) {
				if ($group == $userGroup) {
					WoogleUtil::profileOut(__METHOD__);
					return $group;
				}
			}
		}
		WoogleUtil::profileOut(__METHOD__);
		return '';
	}
	
	/**
	 * Returns one group defined in $config['indexGroups'] which overlaps with the
	 * groups in which the current user is member of 
	 * 
	 * @return String, empty or name of one group
	 */
	public static function getIndexGroup() {
		global $wgUser;
		WoogleUtil::profileIn(__METHOD__);
		// block if user not in one of $groups
		$userGroups = $wgUser->getGroups();
		foreach (self::get('indexGroups') as $group) {
			foreach ($userGroups as $userGroup) {
				if ($group == $userGroup) {
					WoogleUtil::profileOut(__METHOD__);
					return $group;
				}
			}
		}
		WoogleUtil::profileOut(__METHOD__);
		return '';
	}
	
	/**
	 * @return 	boolean, if clickTracking is enabled
	 */
	public static function hasClickTracking() {
		WoogleUtil::profileIn(__METHOD__);
		$res = (self::get('clickTracking') && WoogleConfig::getPref('Privacy', 'TrackUserQueries'));
		return WoogleUtil::profileOut(__METHOD__, $res);
	}
	
	/**
	 * This requires an own method, since otherwise endless recursion
	 * occurs when WoogleUtil::profile... methods are using this..
	 * 
	 * @return boolean, if profiling is enabled
	 */
	public static function useProfiling(){
		return self::$config['useProfiling'][0];
	}	
	
}
