<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

/**
 * 
 * @param $texts   a more or less long string, containing the value of the title attributes of a link. 
 * 					if there are more links, they are separated by ";;"
 * @return the html-code, for every "Tooltip", separated by ";;"
 */
function woogleAjaxRedLinkText($texts, $pageTitle) {
	WoogleUtil::profileIn(__FUNCTION__);
	$qStart = microtime(true);
	
	// encode the links
	$texts = urldecode($texts) ;
	
	// encode the page-title
	$pageTitle = urldecode($pageTitle) ;
	
	// former found in woogle_redlink.js.php
	// getting the new localized "title"-tag for the results
	$rtitle		= wfMsg('red-link-title'); //  (page does not exist) vs.  (not yet written)
	$rtitle		= str_replace('$1', '', $rtitle);

	// Save links in array
	$links = explode(';;', $texts);	
	WoogleUtil::info(print_r($links, true));
	$numRedLinks = count($links);
	$results = '';
	
	foreach( $links as $link) {
		// encode the links
		$link = urldecode($link) ;
		
		$link_rtitle = $link;
		$strpos = strpos($link, $rtitle);
		if( !$strpos ) {				
			WoogleUtil::debug('Title not interesting: '.$link);								
			continue;	// skip this item
		} else {				
			$results .= empty($results) ? '' : ';;';
			// cut the "page does not exist"-string away
			$link = substr( $link, 0, $strpos);		
			// work with data - get an array returnde
			$result = WoogleRedLinks::getRedLinkInfo($link, $pageTitle);
			// urlencode to avoid problems with encoding throw ajax-response
			WoogleUtil::info('Single Result: '.$result[0]);
			$result[0] = urlencode( $result[0] );
			$result[1] = urlencode( $result[1] );
			$single_result = urlencode($link_rtitle).';'.implode(';',$result); 			
			$results .= $single_result;
			// woogleUtil::info('Single Result: '.$single_result);				
		}		
	}	
	
	//$numRedLinks
	$qStop = microtime(true);
	$qTime = $qStop - $qStart;
	WoogleUtil::perflog($qTime . ';' . $numRedLinks, 'redlinks');
	WoogleUtil::profileOut(__FUNCTION__);
	
	return urlencode($rtitle).';;'.$results;
}

/**
 * Is been called by woogle_redlink.js , messures time the user is over an RedLink
 * 
 * @param $link			The Link, on which the user was
 * @param $text			Text, which will be submitted
 * @param $time			The time over the link
 */
function woogleAjaxRedLinkVisit($link, $text, $time) {
	$link = urldecode($link);
	$text = urldecode($text);
	$stringToCut = '&action=edit';
	$pos = strpos($link, $stringToCut);
	$link = substr( $link, 0, $pos);
	WoogleUtil::debug('RedLinkHover' . $text.'/Link = ' . $link. '/Zeit = '.$time);
	wfRunHooks('WoogleRedLinkHover', array($link, $text, $time));
	return '';
}

function woogleSaveInPlace( $title, $text, $type) {
	global $wgOut;
	
	$t = new Title();
	$isDescription = false;
	if( $type = 'discussion') {
		$tt 	= $t->newFromDBkey($title);
	} else if( $type = 'description') {
		$tt 	= $t->newFromDBkey($title);
		$isDescription = true;	
	} else {
		WoogleUtil::debug('WoogleFunctions:woogleSaveInPlace - Type Error: ' . $type);
	}
	
	$article = new Article($tt);
	// will also call add/edit hooks!
	$article->doEdit($text, 'Woogle Page Ajax Edit');	

	$parseOptions = new ParserOptions();
	return $wgOut->parse( $text );	
}

function woogleAjaxAutocomplete($value, $resultNum = 10) {
	
	WoogleUtil::debug('encodierte URL: '.$value);
	$value = urldecode($value);
	WoogleUtil::debug('decodierte URL: '.$value);
	
	$MAX_STRING_LENGTH = 20;
		
	try {				
		WoogleUtil::debug('woogleAjaxAutocomplete called with: ' . $value);
		WoogleUtil::debug('woogleAjaxAutocomplete called with (dec): ' . utf8_decode($value));
		WoogleUtil::debug('woogleAjaxAutocomplete called with (end): ' . utf8_encode($value));
		$value = utf8_encode($value);
		$value = mb_strtolower($value);
		WoogleUtil::debug('woogleAjaxAutocomplete called with (end) and mb: ' . $value);
		
		$termFreqs = WoogleUtil::getQueryService()->getAutoComplete($value, $resultNum);		
	} catch (Exception $e) {
		WoogleUtil::error("WoogleFunctions.php -> woogleAjaxAutocomplete" . $e->getMessage());
		exit;
	}

	$results = '';
	$hits_text = WoogleUtil::getMsg('WoogleAutocompleteHits');
	$hits_text_single = WoogleUtil::getMsg('WoogleAutocompleteHitsSingle');
	
	if($termFreqs) {		
		$i = 0;
		$html = '<ul>';
		foreach ($termFreqs as $tf) {
			$term = $tf['entity'];
			$freq = $tf['freq'] ;
			
			if( strlen($term) > $MAX_STRING_LENGTH ) {
				$term_cutted = substr($term, 0 , $MAX_STRING_LENGTH-1).'...';
			} else {
				$term_cutted = $term;
			}
			
			$term_strong = str_replace($value, '<strong>'.$value.'</strong>',$term_cutted);				

			if($i%2) {				
				$class = 'ac_even';			
			} else {
				$class = 'ac_odd';
			}			
			$image = '<img src="'.WoogleUtil::getImagePath() . 'social_search/promote_result.png" height="8" width="8"/>';
			$link = '<a href="Woogle:'.urlencode($term).'">'.$term_strong.'</a>';			
			if( $freq == 1) {
				$this_hits_text = $hits_text_single;				
			} else {
				$this_hits_text = $hits_text;
			}			
			$html .= '<li class="'.$class.'">'.$image.' '.$link.' <span>('.$freq.' '.$this_hits_text.')</span></li>';
			$i++;		
		}	
		$html .= '</ul>';		
		return $html;
	} elseif( WoogleConfig::get('autoCompleteDebug')) {
		$html = '<ul><li>Autocomplete was called</li></ul>';
		return $html;
	} else {
		return '';		
	}	
}

function woogleAjaxAutocompleteRedirect($query) {		
	
	return WoogleUtil::getWoogleLinkPrefix() . $query;
}

function woogleAjaxAutocompleteAddResult( $value , $resultNum = 10 ) {
	
	$value = urldecode($value);	
		
	try {		
		$value = utf8_encode($value);
		$value = mb_strtolower($value);
		$value .= '*';
		$advanced = WoogleQueryService::getDummyValues();		
		
		$results = WoogleUtil::getQueryService()->executeKeywordQuery($value, $resultNum, 0, $advanced);			
		
	} catch (Exception $e) {
		WoogleUtil::error("WoogleFunctions.php -> woogleAjaxAutocompleteAddResult" . $e->getMessage());
		exit;
	}
	
	if($results['numHits'] > 0) {
		$i = 0;
		$html = '<ul>';
		foreach($results['results'] as $result) {
			// TODO multibyte?
			$label = str_replace($value, '<strong>'.$value.'</strong>',$result['label']);
			$content = str_replace($value, '<strong>'.$value.'</strong>',$result['content']);			
			$img = '<img src="'.WoogleUtil::getWooglePath().'resources/img/icon/'.$result['mimetype'].'.gif" 
					width="12" height="12" onError="this.src=\''.WoogleUtil::getWooglePath() . 'resources/img/icon/unknown.gif\';" />'; 
			
			if($i%2) {				
				$class = 'ac_even';			
			} else {
				$class = 'ac_odd';
			}

			$html .= 
				'<li class="'.$class.'">
					<table>
						<tr>
							<td>'.$img.'</td>
							<td><i>'.$label.'</i></td>
						</tr>
						<tr>
							<td><input type="hidden" value="'.$result['id'].'"/></td>
							<td style="font-size: 0.9em">'.$content.'</td>
						</tr>
					</table>
					<input class="result_title" type="hidden" value="'.$result['label'].'" \>
					<input class="result_id" type="hidden" value="'.$result['id'].'" \>
					<input class="result_uri" type="hidden" value="'.$result['uri'].'" \>
				</li>';
			// $link = '<a href="Woogle:'.urlencode(utf8_decode($term)).'">'.$term_strong.'</a>';			

			// $html .= '<li class="'.$class.'">'.$image.' '.$link.' <span>('.$freq.' '.$this_hits_text.')</span></li>';
			$i++;		
		}		
		$html .= '</ul>';
		WoogleUtil::info('HTML: '.$html);
		return $html;
	} else {
		return '';	
	}	
}

function wooglePromoteResult($query, $result_url) {
	WoogleUtil::error($result_url.' (promote) for query: '.$query);
	return true;
}

function woogleDegradeResult($query, $result_url) {
	WoogleUtil::error($result_url.' (degrade) for query: '.$query);
	return true;
}

function woogleRestoreResult($query, $result_url) {
	WoogleUtil::error($result_url.' (degrade) for query: '.$query);
	return true;
}

function woogleRemoveResult($query, $result_url) {
	WoogleUtil::error($result_url.' (degrade) for query: '.$query);
	return true;
}

function woogleAddComment($query, $result_url, $comment_text) {
	// TODO: adding DB entries for this comment

	$html = '<div class="comment" style="display:none">'.$comment_text.'</div>';
	return $html;
}

function woogleAddResult($query, $url, $result_id, $title = false, $description = false) {
	
	//TODO: DB-stuff
	
	//Preparing HTML for newly added Result
	$head = '';
	$content = '';
	
	if( $title ) {
		$linktitle = $title;
	} else {
		$linktitle = $url;
	}
	if( $description ) {
		$content = $description.'<br/>';
	}
	$html = '<div style="display: none" class="woogle_result">'	
			.'<span class="woogle_resultlink">'
			.'<a rel="nofollow" href="'.$url.'">'.$linktitle.'</a>'
			.'</span><br />'
			.$content
			.WoogleSearch::getNewChangeButtons().'<br />'
			.'</div>';	
			
	return $html;
}
/**
 * Woogle uses autoloading instead of explicitly 'including' other .php files
 * http://de3.php.net/__autoload
 * 
 * This method writes the Woogle-specific classes in the global MediaWiki
 * autoload array
 * 
 * @param 	$classes	Array of class names as defined in Woogle.php
 * @return 	void
 */
function woogleAutoloadClasses($classes) {
	global $wgAutoloadClasses;
	WoogleUtil::profileIn(__FUNCTION__);
	foreach ($classes as $className => $path) {
		$wgAutoloadClasses[$className] = dirname(__FILE__) . '/..' . $path . '/' . $className . '.php';
	}
	WoogleUtil::profileOut(__FUNCTION__);
}

/**
 * @param  $conf  Array of configuration parameters
 * @return void
 */
function woogleConfigureAddon($conf){
	WoogleUtil::profileIn(__FUNCTION__);
	foreach ($conf as $key => $val){
		// do not set if the key to be set is indexAddon AND has already been defined
		if (! ($key == 'indexAddon') && (WoogleConfig::get('indexAddon', false) != '') ){
			WoogleConfig::setNewValue($key, $val);
		}
	}
	WoogleUtil::profileIn(__FUNCTION__);
}

/**
 * Initialize custom MediaWiki Namespaces required for Woogle
 * http://www.mediawiki.org/wiki/Manual:$wgExtraNamespaces
 * @return void
 */
function woogleInitNamespaces(){
	global $wooNamespaceIndex, $wgExtraNamespaces;
	
	// if namespace shall be different from default, $wooNamespaceIndex
	// has to be set in LocalSettings.php before including Woogle.php
	if (!isset($wooNamespaceIndex)) {
		// if not defined, take default
		$wooNamespaceIndex = WoogleConfig::get('namespaceIndex', false);
	}
	
	define('WOO_NS_MAIN', $wooNamespaceIndex);
	define('WOO_NS_TALK', $wooNamespaceIndex+1);
	
	// Register namespace identifiers
	if (!is_array($wgExtraNamespaces)) { $wgExtraNamespaces=array(); }
	$wgExtraNamespaces = $wgExtraNamespaces + WoogleUtil::getNamespaces();
	
}

/**
 * Loads Woogle addons (=modular extensions to Woogle itself) placed in the \addons directory
 * Addons have to contain a file addon-name.php in the directoy \addons\addon-name\ 
 * 
 * @return 	void
 */
function woogleLoadAddons() {
	WoogleUtil::profileIn(__FUNCTION__);
	
	// check if addons directory exists
	if (file_exists(dirname(__FILE__) . '/..' . '/addons') && is_dir(dirname(__FILE__) . '/..' . '/addons')) {
		$woogleAddons = opendir(dirname(__FILE__) . '/..' . '/addons');

		// read subdirectories
		while (false !== ($file = readdir($woogleAddons))) {
			if (substr($file, 0, 1) != '.' && is_dir(dirname(__FILE__) . '/..' . '/addons/' . $file) && file_exists(dirname(__FILE__) . '/..' . '/addons/' . $file . '/' . $file . '.php')) {
				require_once(dirname(__FILE__) . '/..' . '/addons/' . $file . '/' . $file . '.php');
			}
		}
		closedir($woogleAddons);
	}
	
	WoogleUtil::profileOut(__FUNCTION__);
}

/**
 * Registers special pages defined by Woogle with the core MediaWiki special pages array
 * @return	void
 */
function woogleLoadSpecialPages() {
	global $wgSpecialPages;
	WoogleUtil::profileIn(__FUNCTION__);
	foreach (WoogleUtil::getSpecialPages() as $specialPage) {
		$wgSpecialPages[WoogleUtil::getSpecialPageClass($specialPage)] = WoogleUtil::getSpecialPageClass($specialPage);
	}
	WoogleUtil::profileOut(__FUNCTION__);
}


if ( !function_exists( 'mb_stripos' ) ) {
	function mb_stripos( $hackstack, $needle, $offset=0, $encoding='' ) {
		// delegate to allow for testing fallback method even if regular mb_stripos exsits!
		return mb_stripos_fallback( $hackstack, $needle, $offset, $encoding);
	}
}

/**
  * Fallback implementation for mb_stripos which exsits only since PHP 5 >= 5.2.0 
  * (http://de3.php.net/manual/en/function.mb-stripos.php)
  */
function mb_stripos_fallback( $hackstack, $needle, $offset=0, $encoding='' ) {
	// int mb_stripos  ( string $haystack  , string $needle  [, int $offset  [, string $encoding  ]] )
	// int mb_strpos  ( string $haystack  , string $needle  [, int $offset  [, string $encoding  ]] )
	// string mb_strtolower  ( string $str  [, string $encoding = mb_internal_encoding()  ] )
	WoogleUtil::profileIn(__FUNCTION__);
	if ($encoding != ''){
		return mb_strpos( mb_strtolower($hackstack, $encoding), mb_strtolower($needle, $encoding), $offset, $encoding );
	} else {
		return mb_strpos( mb_strtolower($hackstack), mb_strtolower($needle), $offset);
	}
	WoogleUtil::profileOut(__FUNCTION__);
	return $res;
}
