<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

/**
 *	This file lists all MediaWiki hooks (http://www.mediawiki.org/wiki/Manual:Hooks) consumed
 *  by the Woogle core code
 *  
 *  Each hook is assigned a callback function in this file, which will be called at runtime if
 *  the hook fires.
 *  
 *  Besides the hooks defined by MediaWiki, Woogle itself offers a number of hooks, which are
 *  documented below.
 */

/*
own Hooks
* listed by file in which they are defined
* specifying a) the name of the hook, b) the name of its parameters and c) the Woogle version it was introduced

WoogleConfig.php:
	WoogleConfig			&$config				// Woogle >= 0.99
	WoogleMessages			&$messages				// Woogle >= 0.99
	WooglePreferences		&$preferences			// Woogle >= 0.99

WoogleSearch.php:
	WoogleSearchInit		$woogleSearch			// Woogle >= 0.99
	
redirect.php:
	WoogleResultClick		$url, $query, $pos		// Woogle >= 0.99
	
WoogleHooks.php:
	WooglePreferencesSave	$name, $value			// Woogle >= 0.99	
	
WoogleSpecialHelp.php:
	WoogleHelpInit			$wgOut					// Woogle >= 1.0-RC2
	WoogleHelpExit			$wgOut					// Woogle >= 1.0-RC2
	
*/

// MediaWiki hook									callback function (see below)					available since
$wgHooks['ArticleDelete'][]							= 'woogleArticleDelete';						// MW >= 1.4.0
$wgHooks['ArticleDeleteComplete'][]					= 'woogleArticleDeleteComplete';				// MW >= 1.4.0
$wgHooks['ArticleFromTitle'][]						= 'woogleArticleFromTitle';						// MW >= 1.8.0
$wgHooks['ArticleInsertComplete'][]					= 'woogleArticleInsertComplete';				// MW >= 1.6.0
$wgHooks['ArticleSaveComplete'][]					= 'woogleArticleSaveComplete';					// MW >= 1.4.0
$wgHooks['CustomEditor'][]							= 'woogleCustomEditor';							// MW >= 1.9.1
$wgHooks['LangugeGetSpecialPageAliases'][]			= 'woogleLanguageGetSpecialPageAliases';		// MW >= 1.9.0 && MW <= 1.11.x
$wgHooks['LanguageGetSpecialPageAliases'][]			= 'woogleLanguageGetSpecialPageAliases';		// MW >= 1.12.0
$wgHooks['LoadAllMessages'][]						= 'woogleLoadAllMessages';						// MW >= 1.8.0
$wgHooks['OutputPageBeforeHTML'][]					= 'woogleOutputPageBeforeHTML';					// MW >= 1.6.0
$wgHooks['RenderPreferencesForm'][]					= 'woogleRenderPreferencesForm';				// MW >= 1.11.0 (deprecated in 1.16.0)
$wgHooks['GetPreferences'][] 						= 'woogleGetPreferences';						// MW >= 1.16.0
$wgHooks['SavePreferences'][]						= 'woogleSavePreferences';						// MW >= 1.11.0 (deprecated in 1.16.0)
$wgHooks['UserSaveOptions'][] 						= 'woogleUserSaveOptions';						// MW >= 1.16.0

//$wgHooks['SkinTemplatePreventOtherActiveTabs'][]	= 'woogleSkinTemplatePreventOtherActiveTabs';	// MW >= 1.6.0
$wgHooks['SpecialMovepageAfterMove'][]				= 'woogleSpecialMovepageAfterMove';				// MW >= 1.6.0
$wgHooks['SpecialPageExecuteBeforePage'][]			= 'woogleSpecialPageExecuteBeforePage';			// MW >= 1.6.0 (deprecated in 1.14.0)
$wgHooks['SpecialPage_initList'][] 					= 'woogleSpecialPageInit';						// MW >= 1.6.0
$wgHooks['AjaxAddScript'][]							= 'woogleAjaxAddScript';						// MW >= 1.9.1

$wgHooks['UploadComplete'][] 						= 'woogleUploadComplete';						// MW >= 1.6.4
$wgHooks['FileUpload'][] 							= 'woogleFileUpload';							// ?
$wgHooks['FileDeleteComplete'][] 					= 'woogleFileDeleteComplete';					// ?

// query logging
$wgHooks['SpecialSearchNoResults'][]				= 'woogleLogNoSearchResults';					// ?
$wgHooks['SpecialSearchResults'][]					= 'woogleLogSearchResults';						// ?
$wgHooks['WoogleSearchLogQuery'][]					= 'woogleLogWoogleQuery';						// ?

// http://www.mediawiki.org/wiki/Manual:Hooks/OpenSearchUrls
// This is only called when open search url is added?
$wgHooks['OpenSearchUrls'][] 						= 'woogleOpenSearch';	// MW >= 1.13.0
function woogleOpenSearch(&$urls) {  
	// TODO
	WoogleUtil::debug("OPEN SEARCH " . print_r($urls, true));
	return true;
}


// fix for MW < 1.13
function woogleArticleDelete(&$article, &$user, &$reason) {
	global $woogleArticleDeleteId;
	WoogleUtil::profileIn(__FUNCTION__);
	$woogleArticleDeleteId = $article->getID();
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

function woogleArticleDeleteComplete(&$article, &$user, $reason, $id = -1) {
	WoogleUtil::profileIn(__FUNCTION__);
	if ($id < 0) {
		global $woogleArticleDeleteId;
		$id = $woogleArticleDeleteId;
	}
	WoogleUtil::getIndexService()->deleteItem($id);
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}


/**
 *	Intercept article creation for Woogle:Pages by instantiating own class 
 */
function woogleArticleFromTitle(&$title, &$article) {
	WoogleUtil::profileIn(__FUNCTION__);
	
	// check if Woogle pages are enabled
	if (WoogleConfig::get('advanced') || WoogleConfig::get('advancedNamespaceGrab')) {
		WoogleUtil::loadMessages();
		
		// check if this is a woogle page call
		if ($title->getNsText() == wfMsg('WoogleSpecialSearch')) {
			if ($title->getText() != ''){
				WoogleUtil::debug("Create Woogle Article");
				$article = new WoogleArticleSearch($title);
			} else {
				// go to Special:Woogle in case of empty call
				WoogleUtil::debug("Empty article title - redirect");
				$wgOut->redirect(WoogleUtil::getWoogleLinkPrefixNonadvanced() . $searchString . $trail);
				WoogleUtil::profileOut(__METHOD__);
				return;
			}
		}
	}
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

function woogleArticleInsertComplete(&$article, &$user, $text, $summary, $minoredit, $watchthis, $sectionanchor, &$flags, $revision) {
	WoogleUtil::profileIn(__FUNCTION__);
	WoogleUtil::getIndexService()->addItem($article->getID());
	if (WoogleConfig::get('watchSearchResults')) WoogleAlertManager::matchDocument(WoogleUtil::articleToDocument($article), WoogleAlertManager::$STATUS_CREATED);
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

function woogleArticleSaveComplete(&$article, &$user, $text, $summary, $minoredit, $watchthis, $sectionanchor, &$flags, $revision) {
	WoogleUtil::profileIn(__FUNCTION__);
	WoogleUtil::getIndexService()->updateItem($article->getID());
	if (WoogleConfig::get('watchSearchResults')) WoogleAlertManager::matchDocument(WoogleUtil::articleToDocument($article), WoogleAlertManager::$STATUS_UPDATED);
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

function woogleUploadComplete( &$image ){
	WoogleUtil::profileIn(__FUNCTION__);
	
	/*
	  http://www.mediawiki.org/wiki/Manual:Hooks/UploadComplete
	  
	  vs.
	  http://www.mediawiki.org/wiki/Manual:Hooks/FileUpload
	  http://www.mediawiki.org/wiki/Manual:Hooks/FileDeleteComplete
	 
	 */
	WoogleUtil::error("Saving file - woogleUploadComplete " . print_r($image, true));	// TODO
	
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

// 1.11.0
function woogleFileUpload( $file){
	WoogleUtil::profileIn(__FUNCTION__);
	WoogleUtil::error("Saving file - woogleFileUpload " . print_r($file, true));// TODO
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

// 1.13.0
function woogleFileDeleteComplete( $file, $oldimage, $article, $user, $reason){
	WoogleUtil::profileIn(__FUNCTION__);
	WoogleUtil::error("Saving file - woogleFileDeleteComplete " . print_r($file, true));// TODO
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}


// http://www.mediawiki.org/wiki/Manual:Hooks/EditFormPreloadText
$wgHooks['EditFormPreloadText'][] 	= 'woogleEditNewPage'; // TODO move up
function woogleEditNewPage(&$text, &$title){
	WoogleUtil::profileIn(__FUNCTION__);
	global $wgRequest, $wgOut;
	
	// only do in advanced mode
	if (WoogleConfig::get('advanced')) {
		// only for Woogle pages
		if (strpos($title->getText() , "Woogle:") === 0){ // TODO check
			// only if src != woo
			if ($wgRequest->getVal('src') != 'woo'){
				$wgOut->redirect(WoogleMWUtil::getArticlePath() . $title);
				WoogleUtil::profileOut(__METHOD__);
				return false;
			}
		}
	}
	
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

/**
 * Renders pages in the "Woogle:" namespace differently (e.g. restricts the size of the edit form
 * 
 * Callback for hook http://www.mediawiki.org/wiki/Manual:Hooks/CustomEditor
 * 
 * @param $article
 * @param $user
 * @return unknown_type
 */
function woogleCustomEditor($article, $user) {
	global $wgDefaultUserOptions;
	WoogleUtil::profileIn(__FUNCTION__);
	WoogleUtil::loadMessages();
	// only do in advanced mode
	if (WoogleConfig::get('advanced')) {
		//WoogleUtil::debug("NST: " . $article->getTitle()->getNsText());
		//WoogleUtil::debug("WF: " . wfMsg('WoogleSpecialSearch'));
		
		// only do if article from Woogle namespace is called
		if ($article->getTitle()->getNsText() == wfMsg('WoogleSpecialSearch')) {
			
			$rows = $user->getOption('rows');
			if ($rows != 3) $user->setOption('rowsOld', $rows);
			WoogleUtil::debug("ROWS1: " . $rows);
			
			$user->setOption('rows', 3);
		} else {
			$rows = $user->getOption('rows');
			WoogleUtil::debug("ROWS2: " . $rows);
			
			if ($rows < 5){
				$rows = $user->getOption('rowsOld', $wgDefaultUserOptions['rows']);
				$user->setOption('rows', $rows);
			}
		}
				
	}
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

function woogleLanguageGetSpecialPageAliases(&$specialPageArray, $code) {
	WoogleUtil::profileIn(__FUNCTION__);
	# The localized title of the special page is among the messages of the
	# extension:
	WoogleUtil::loadMessages();
	foreach (WoogleUtil::getSpecialPages() as $specialPage) {
		$text = WoogleUtil::getSpecialPageLocal($specialPage);
		# Convert from title in text form to DBKey and put it into the alias array:
		$title = Title::newFromText($text);
		$specialPageArray[WoogleUtil::getSpecialPageClass($specialPage)][] = $title->getDBKey();
	}
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

function woogleLoadAllMessages() {
	WoogleUtil::profileIn(__FUNCTION__);
	WoogleUtil::loadMessages();
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

/*
 * Load JS/CSS-Files on special Pages
 * ( replaces woogleSpecialPageExecuteBeforePage() )
 */
function woogleAjaxAddScript ( &$out ) {
	global $wgJsMimeType, $woogleHeadersSent;

	// exit if disabled
	if (!WoogleConfig::isEnabled() || !WoogleConfig::hasPermission()) return true;
		
	WoogleUtil::profileIn(__FUNCTION__);
	if (!$woogleHeadersSent) {
		$woogleHeadersSent = true;
		$out->addLink( 
			array( 
				'rel' => 'stylesheet', 
				'type' => 'text/css', 
				'href' => WoogleUtil::getWooglePath() . 'services/woogle.css.php'
			) 
		);	
		
		$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/jQuery/jquery.js"></script>');
		$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'services/woogle.js.php"></script>');
		$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/woogleUtil_lib.js"></script>');			
		// autocompleted Search
		if( WoogleConfig::get('autoComplete')) {			
			$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/woogleUtil_autocomplete.js"></script>');
			$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/woogle_autocomplete.js"></script>');	
		}		
		if( WoogleConfig::get('autoComplete') && WoogleConfig::get('replace')) {
			$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/native_woogle_autocomplete.js"></script>');
		}
	}
		
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
	
}

/*
 * Load JS/CSS-Files on normal Pages
 */
function woogleOutputPageBeforeHTML(&$out, &$text) {
	global $wgJsMimeType, $woogleHeadersSent;
	
	// exit if disabled
	if (!WoogleConfig::isEnabled() || !WoogleConfig::hasPermission()) return true;
	
	WoogleUtil::profileIn(__FUNCTION__);
	if (!$woogleHeadersSent) {
		$woogleHeadersSent = true;
		$woogleHeadersSent = true;
		$out->addLink( 
			array( 
				'rel' => 'stylesheet', 
				'type' => 'text/css', 
				'href' => WoogleUtil::getWooglePath() . 'services/woogle.css.php'
			) 
		);			
			
		$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/jQuery/jquery.js"></script>');
		$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'services/woogle.js.php"></script>');
		$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/woogleUtil_lib.js"></script>');			
				
		// base-plugin, for any kind of autocompletion
		if( WoogleConfig::get('socialResults') || WoogleConfig::get('autoComplete')) {
			$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/woogleUtil_autocomplete.js"></script>');
		}		
		// red link popup java script
		if (WoogleConfig::get('redLinkInfo')){
			$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/woogle_redlink.js"></script>');
		}			
		// autocompleted Search
		if( WoogleConfig::get('autoComplete')) {
			$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/woogle_autocomplete.js"></script>');	
		}
		// autocompleted Search in MediaWiki-SearchBox	
		if( WoogleConfig::get('autoComplete') && WoogleConfig::get('replace')) {
			$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/native_woogle_autocomplete.js"></script>');	
		}	
		// adds SocialResults-Features like add, restore, remove, rank and comment
		if( WoogleConfig::get('socialResults')) {
			$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/woogle_socialResults.js"></script>');	
		}		
		// inPlaceEditing
		if (WoogleConfig::get('inPlaceEditing')){
			$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/woogle_inplace.js"></script>');								
		}
		//	bigImagesOnHover
		if (WoogleConfig::get('bigImagesOnHover')){
			$out->addScript('<script type="' . $wgJsMimeType . '" src="' . WoogleUtil::getWooglePath() . 'resources/js/plugins/woogle_imghover.js"></script>');								
		}
		

	}

	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

/**
 *  Helps rendering Woogle user preferences at Special:Preferences
 * 
 * 	Replaces woogleRenderPreferencesForm() for >= MW 1.16.0
 *  http://www.mediawiki.org/wiki/Manual:Hooks/GetPreferences
 *  
 *  @param $user			User whose preferences are being modified.
 *  @param &$preferences	Preferences description array, to be fed to an HTMLForm object
 *
 */
function woogleGetPreferences($user, &$preferences){
	global $wgDefaultUserOptions; 
	
	// Show preferences only if enabled and for permitted user
	if (!WoogleConfig::get('core') || !WoogleConfig::hasPermission()) return true;

	WoogleUtil::profileIn(__FUNCTION__);
	
	// get user preferences
	$prefs = WoogleConfig::getPreferences();
	
	// iterate preference groups (categories)
	foreach ($prefs as $groupKey => $groupValue) {
	
		// iterate individual preferences
		foreach ($groupValue as $prefKey => $prefValue) {
		
			$type 		= WoogleConfig::getPreferenceType($groupKey, $prefKey);
			$default 	= WoogleConfig::getPreferenceDefault($groupKey, $prefKey);
			$name 		= WoogleConfig::getPreferenceName($groupKey, $prefKey);
			//$value 		= $user->getOption($name, $default); // TODO: default/value not used!
		
			$preferences[$name] = array(
				'type'		=> $type,
				'label'		=> wfMsg('wooglePref' . $groupKey . $prefKey),
				//'help'		=> 'help',	 // TODO add help text
				//'help-message' => '',
				'section'	=> 'wooglePreferencesMain/wooglePreferencesGroups' . $groupKey, 
				//'default' 	=> $default,	// DOES HARM if enabled!
			);
		
			switch ($type) {
				case 'text':
					$preferences[$name]['class'] = 'HTMLTextField';
					break;
				case 'text-nonempty':
					$preferences[$name]['class'] = 'HTMLTextField';
					$preferences[$name]['options'] = array('required' => true); // TODO does not work yet?
					$preferences[$name]['required'] = true;
					break;
				case 'checkbox':
					$preferences[$name]['class'] = 'HTMLCheckField';
					break;
				case 'label':
					// TODO this should probably not be handled als label?
					//$preferences[$name]['class'] = 'HTMLInfoField';
					$preferences[$name]['class'] = 'HTMLTextField';
					$preferences[$name]['disabled'] = true;
					break;
			}
			
			// Show default options in Special:Preferences
			// adapted from http://svn.wikimedia.org/viewvc/mediawiki/trunk/extensions/FCKeditor/FCKeditor.body.php
			if( !array_key_exists( $name, $user->mOptions ) && !empty( $default ) )
				$user->setOption( $name, $default ); 
			
			
		}
	}
	
	// add hidden field for managing woogleUserSaveOptions
	$preferences['WooglePreferencesHiddenKey'] = array(
		'type'		=>	'info',
		'class'		=>  'HTMLHiddenField',
		'default'	=>	'woogleprefsave',
	);
	
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}


/**
 * For < MW 1.16.0 compatibility, later replaced by woogleGetPreferences()
 */
function woogleRenderPreferencesForm($specialPreferences, $out) {
	global $wgUser;
	
	// Show preferences only if enabled and for permitted user
	if (!WoogleConfig::get('core') || !WoogleConfig::hasPermission()) return true;
	
	WoogleUtil::profileIn(__FUNCTION__);
	
	$out->addHTML('<fieldset><legend>Woogle</legend>'); // Tab title
	
	// get user preferences
	$preferences = WoogleConfig::getPreferences();
	
	// iterate preference groups (categories)
	foreach ($preferences as $groupKey => $groupValue) {
		$out->addHTML('<fieldset><legend>' . WoogleUtil::getMsg('PreferencesGroups' . $groupKey ) . '</legend>');
		$out->addHTML('<table>');
		
		// iterate individual preferences
		foreach ($groupValue as $prefKey => $prefValue) {
			$type = WoogleConfig::getPreferenceType($groupKey, $prefKey);
			$default = WoogleConfig::getPreferenceDefault($groupKey, $prefKey);
			$name = WoogleConfig::getPreferenceName($groupKey, $prefKey);
			$value = $wgUser->getOption($name, $default);
			switch ($type) {
				case 'text':
				case 'text-nonempty':
					$out->addHTML($specialPreferences->addRow(wfMsg('wooglePref' . $groupKey . $prefKey) . ': ',
						'<input id="' . $name . '" name="wp' . $name . '" value="' . $value . '" size="' . max(20, round(strlen($value) * 1.3)) . '" />'));
					break;
				case 'checkbox':
					$out->addHTML($specialPreferences->addRow('<span class="toggletext"><label for="' . $name . '">' . wfMsg('wooglePref' . $groupKey . $prefKey) . '</label>: </span>',
						'<input type="checkbox" value="1" id="' . $name . '" name="wp' . $name . '"' . ($value == 1 ? ' checked="checked"' : '') . ' />'));
					break;
				case 'label':
					$out->addHTML($specialPreferences->addRow(wfMsg('wooglePref' . $groupKey . $prefKey) . ': ', $value));
					break;
			}
		}
		$out->addHTML('</table></fieldset>');
	}
	
	$out->addHTML('</fieldset>');
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

/**
 *  Called before saving user preferences (changed via Special:Preferences)
 *  and also for each explicit $user->saveOptions() call - also in the Woogle code
 *
 *	Since 1.16.0 preferences are saved automatically. Main purpose of implementing
 *  this hook is thus to recognized changed Woogle preferences for instrumentation
 *  
 * 	Replaces woogleSavePreferences() for >= MW 1.16.0
 *  http://www.mediawiki.org/wiki/Manual:Hooks/UserSaveOptions
 *  
 *  @param $user		User object
 *  @param &$options	Options, can be modified. ($options['WoogleInstInstGroup'] = 'Sierra')
 *  
 */
function woogleUserSaveOptions($user, &$options){
	global $wgRequest;
	WoogleUtil::profileIn(__FUNCTION__);

	// exit if call does not come from preferences form
	if (!array_key_exists('WooglePreferencesHiddenKey', $options) || !$options['WooglePreferencesHiddenKey'] == 'woogleprefsave') return true;
	
	WoogleUtil::debug('woogleUserSaveOptions:otions ' . print_r($options, true));
	
	unset($options['WooglePreferencesHiddenKey']); // avoid saving key to db
	
	// get user preferences
	$preferences = WoogleConfig::getPreferences();
	
	// iterate preference groups (categories)
	foreach ($preferences as $groupKey => $groupValue) {
		
		// iterate individual preferences
		foreach ($groupValue as $prefKey => $prefValue) {
			
			$name = WoogleConfig::getPreferenceName($groupKey, $prefKey);
			$value = $options[$name];
			
			// TODO strange enough, checkbox properties need to be saved explicitly if disabled
			if (WoogleConfig::getPreferenceType($groupKey, $prefKey) == 'checkbox'){
				WoogleUtil::debug('woogleUserSaveOptions - value for ' . $name . ' is ' . $value);
				if (empty($value)) $value = 0;
				//$user->setOption($name, $value);
				$options[$name] = $value;
			}			
			
			// allow other parts of Woogle (e.g. instrumentation) to recognize changed values
			wfRunHooks('WooglePreferencesSave', array($name, $value));
	
		}
	}
		
	
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

/**
 * For < MW 1.16.0 compatibility, later replaced by woogleUserSaveOptions()
 * 
 * Note that effectively, only the $user variable is used from the set of parameters
 */
function woogleSavePreferences($specialPreferences, $user, &$msg, $oldOptions) {
	global $wgRequest;
	WoogleUtil::profileIn(__FUNCTION__);
	
	// get user preferences
	$preferences = WoogleConfig::getPreferences();
	
	// iterate preference groups (categories)
	foreach ($preferences as $groupKey => $groupValue) {
		
		// iterate individual preferences
		foreach ($groupValue as $prefKey => $prefValue) {
			
			$name = WoogleConfig::getPreferenceName($groupKey, $prefKey);
			$value = $wgRequest->getVal('wp' . $name);
			
			// do not save/change label values that are only displayed
			if (WoogleConfig::getPreferenceType($groupKey, $prefKey) != 'label'){
			
				// randomID is e.g. a text-nonempty value. It should only be overwritten, if a value is set			
				if (WoogleConfig::getPreferenceType($groupKey, $prefKey) == 'text-nonempty' && empty($value)){
					// do not allow to save empty value
					WoogleUtil::info('Prohibited to save empty preference value for ' . $name);
				} else {
					$user->setOption($name, $value);
					// allow other parts of Woogle (e.g. instrumentation) to recognize changed values
					wfRunHooks('WooglePreferencesSave', array($name, $value));
				}
				
			}
		}
	}
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

function woogleSkinTemplatePreventOtherActiveTabs(&$skin, &$prevent_active_tabs) {
	WoogleUtil::profileIn(__FUNCTION__);
	WoogleUtil::loadMessages();
	$titleParts = explode(':', $skin->mTitle);
	if ($titleParts[0] == wfMsg('WoogleSpecialSearch')) {
		$skin->iscontent = false;
	}
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

function woogleSpecialMovepageAfterMove(&$article, &$ot, &$nt) {
	WoogleUtil::profileIn(__FUNCTION__);
	// update old page, as it's not deleted but is redirecting to the new page
	WoogleUtil::getIndexService()->updateItem($ot->getArticleID());
	WoogleUtil::getIndexService()->addItem($nt->getArticleID());
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

/*
 * Deprecated since MediaWiki 1.14
 * alternative: woogleAjaxAddScript
 * 
 */
function woogleSpecialPageExecuteBeforePage($specialpage, $parameters, $function) {
	global $wgOut;
	WoogleUtil::profileIn(__FUNCTION__);
	$headers = woogleAjaxAddScript($wgOut);
	WoogleUtil::profileOut(__FUNCTION__);
	return $headers;	
}


/**
 * c.f. http://www.mediawiki.org/wiki/Manual:Hooks/SpecialPage_initList
 * @param	$aSpecialPages
 * @return 	boolean
 */
function woogleSpecialPageInit(&$aSpecialPages) {
	global $wgSpecialPages;
	
	// Replace standard internal MediaWiki search (Special:Search) with Woogle if this is configured
	$usewikitmp = false;
	if (isset($_REQUEST['usewikisearch']) && $_REQUEST['usewikisearch'] == 1) $usewikitmp = true;
	
	if (WoogleConfig::isEnabled() && WoogleConfig::isReplacingMWS() && WoogleConfig::hasPermission() && !$usewikitmp){
		global $wgDisableInternalSearch;
		$wgDisableInternalSearch = true;
		$aSpecialPages['Search'] = WoogleUtil::getSpecialPageClass('DummySearch');
	}
	return true;
}

function woogleLogNoSearchResults($term) { 
	WoogleUtil::profileIn(__FUNCTION__);
	if (WoogleConfig::get('logQuery')) WoogleUtil::log_query('mwsearch_noresults;' . $term);
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

function woogleLogSearchResults($term, &$titleMatches, &$textMatches) {
	WoogleUtil::profileIn(__FUNCTION__);
	if (WoogleConfig::get('logQuery')){
		$titleNum = -1;
		$textNum = -1;
		if (isset($titleMatches)) $titleNum = $titleMatches->numRows();
		if (isset($textMatches)) $textNum = $textMatches->numRows();
		WoogleUtil::log_query('mwsearch_results;' . $term . ';' . $titleNum . ';' . $textNum);
	}
	WoogleUtil::profileOut(__FUNCTION__);
	return true;
}

function woogleLogWoogleQuery($queryString){
	WoogleUtil::profileIn(__FUNCTION__);
	if (WoogleConfig::get('logQuery')) WoogleUtil::log_query('woogle_search_query;' . $queryString . ';UGT:' . str_replace( ';' , '-', $_SERVER['HTTP_USER_AGENT']));
	WoogleUtil::profileOut(__FUNCTION__);
	return true;	
}

