<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

abstract class WoogleIndexService {

	/**
	 * checks if indexing connection works
	 * @return boolean
	 */
	abstract public function checkIndexService();
	
			
	/**
	 * @return			string, the location of the index (or url or whatever)
	 */
	abstract public function getLocation();
	
	
	/**
	 * @return			int, how many documents are in the index
	 */
	abstract public function numDocs();
	
	/**
	 * @param $id		int, the id of the article to be searched in the index
	 * @return			bool, whether the article with the given id was found in the index
	 */
	abstract public function exists($id);
	
	/**
	 * @return			float, cumulated size of all files in the index directory in kilobytes
	 */
	abstract public function getIndexFileSize();
	

	abstract protected function optimizeIndex();
	abstract protected function startReIndex();
	abstract protected function stopReIndex();
	
	/**
	 * @param $article	WoogleDocument, to be updated in the index
	 * @return			void
	 */
	abstract public function doUpdateItem($wd);
	
	/**
	 * @param $id		id of article to be deleted from the index
	 * @return			void
	 */
	abstract public function doDeleteItem($id);
	
	/**
	 * @param $article	WoogleDocument, to be added to the index
	 * @param $flush	if index should be flushed immediately
	 * @return			void
	 */
	abstract public function doAddItem($wd, $flush=true);
	
	/**
	 * @param $article		array of WoogleDocuments to be added to the index
	 * @param $flush		boolean, if index should be flushed immediately
	 * @param $optimize		boolean, if the index should be optimized afterwards
	 * @param $printStatus	boolean, if to print status information to the command line
	 * @return			void
	 */
	abstract public function doAddItems($wda, $flush=true, $optimize=false, $printStatus=false);
	
	
	private function addItems($wda){
		WoogleUtil::profileIn(__METHOD__);
		foreach($wda as $wd){
			WoogleJobRefreshItem::addJob($wd->getShortId());
		}
		WoogleUtil::profileOut(__METHOD__);
	}
	

	public function addItem($id) {
		WoogleUtil::profileIn(__METHOD__);
		// TODO disabled jobs due to http://octopus13.fzi.de:8080/browse/WOO-416 
		//		partial re-enabled due to WOO-443
		
		// Use Job Indexing for everything except Ajax requests
		global $wgRequest;
		if ($wgRequest->getText( 'action' )!='ajax'){
			WoogleJobRefreshItem::addJob($id);
			return;
		}
	
		$title = Title::newFromID($id);
		$article = new Article($title, 0);
		
		// omit excluded namespaces
		$namespaces = WoogleConfig::get('excludeNamespaces');
		if (in_array($title->getNamespace(), $namespaces)){
			WoogleUtil::debug('WoogleIndexService:addItem() skipping ' . $id);
			return WoogleUtil::profileOut(__METHOD__);
		}
			
		// do not index NS_FILE article pages, if file content indexing is enabled
		if (WoogleConfig::get('indexFileContent') && ($title->getNamespace() == NS_FILE)){
			WoogleUtil::debug('WoogleIndexService:addItem() skipping ' . $id);
			return WoogleUtil::profileOut(__METHOD__);
		}
		
		$this->doAddItem(WoogleUtil::articleToDocument($article));
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function updateItem($id) {
		WoogleUtil::profileIn(__METHOD__);
		// TODO disabled jobs due to http://octopus13.fzi.de:8080/browse/WOO-416
		//		partial re-enabled due to WOO-443
		
		// Use Job Indexing for everything except Ajax requests
		global $wgRequest;
		if ($wgRequest->getText( 'action' )!='ajax'){
			WoogleJobRefreshItem::addJob($id);
			return;
		}

		$title = Title::newFromID($id);
		$article = new Article($title, 0);
		
		// omit excluded namespaces
		$namespaces = WoogleConfig::get('excludeNamespaces');
		if (in_array($title->getNamespace(), $namespaces)){
			WoogleUtil::debug('WoogleIndexService:updateItem() skipping ' . $id);
			return WoogleUtil::profileOut(__METHOD__);
		}
			
		// do not index NS_FILE article pages, if file content indexing is enabled
		if (WoogleConfig::get('indexFileContent') && ($title->getNamespace() == NS_FILE)){
			WoogleUtil::debug('WoogleIndexService:updateItem() skipping ' . $id);
			return WoogleUtil::profileOut(__METHOD__);
		}
		
		$this->doUpdateItem(WoogleUtil::articleToDocument($article));
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function deleteItem($id) {
		WoogleUtil::profileIn(__METHOD__);
		// TODO disabled jobs due to http://octopus13.fzi.de:8080/browse/WOO-416
		//		partial re-enabled due to WOO-443
		
		// Use Job Indexing for everything except Ajax requests
		global $wgRequest;
		if ($wgRequest->getText( 'action' )!='ajax'){
			WoogleJobRefreshItem::addJob($id);
			return;
		}
		
		$title = Title::newFromID($id);
		
		// TODO this causes a bug (http://octopus13.fzi.de:8080/browse/WOO-436)
		/* do not index NS < 0 and NS_MEDIAWIKI
		if (($title->getNamespace() < 0) || ($title->getNamespace() == NS_MEDIAWIKI) ){
			WoogleUtil::debug('WoogleIndexService:deleteItem() skipping ' . $id);
			return WoogleUtil::profileOut(__METHOD__);
		}
			
		// do not index Media-NS article pages, if file content indexing is enabled
		if (WoogleConfig::get('indexFileContent') && ($title->getNamespace() == NS_IMAGE)){ // TODO namespace
			WoogleUtil::debug('WoogleIndexService:deleteItem() skipping ' . $id);
			return WoogleUtil::profileOut(__METHOD__);
		}*/		
		
		$this->doDeleteItem($id);
		WoogleUtil::profileOut(__METHOD__);
	}

	/**
	 * (re-)indexes all wiki pages  
	 * 
	 * @param $direct		boolean, if re-indexing should be direct (=during one http request or via the job API 
	 * @param $printStatus	boolean, if to print status information to the command line
	 * @return 				void
	 */
	public function index($direct = true, $printStatus = false){
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		
		$wgOut->addHTML('indexing all wiki pages...<br />');
		$this->startReIndex(); // will empty existing index

		//
		// Query articles via Db
		//

		// store the starting time
		$startTime = time();
		$wgOut->addHTML('start time: ' . date('c', $startTime) . '<br />');

		// Find titles in database
		if ($printStatus) echo "Querying pages from the database";

		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('page', 'text', 'revision'));
		$sql_add = '';
		
		// omit files if files are indexed separately (below)
		if (WoogleConfig::get('indexFileContent')){
			$sql_add = ' AND pg.page_namespace != ' . NS_FILE . ' ';
		}
		
		$sql = 'SELECT pg.page_id AS pid, pg.page_title AS title, pg.page_touched AS touched, pg.page_namespace AS nspace, tx.old_text AS txt ' .
		'FROM ' . $page . ' pg INNER JOIN ' . $revision . ' rev ON pg.page_latest = rev.rev_id INNER JOIN ' . $text . ' tx ON  rev.rev_text_id = tx.old_id WHERE pg.page_namespace >= 0';

		// exclude namespaces
		$namespaces = WoogleConfig::get('excludeNamespaces');
		foreach($namespaces as $nsId){			
			$sql .= ' AND pg.page_namespace != ' . $nsId . ' ';
		}
		$sql .= ' ' . $sql_add . ';';
		$res = $db->doQuery($sql);
		
		// now only consider positive namespaces, negative ones hold special pages and links to media files
		
		$docs = array();
		while ($row = $db->fetchObject($res)) {

			$ptitle = $row->title;
			if ($row->nspace != 0){
				$nspace = MWNamespace::getCanonicalName($row->nspace);
				if ($nspace != false) $ptitle = $nspace . ':' . $ptitle;
			}
			$wd = new WoogleDocument();
			$wd->setId(WoogleQueryServiceCommon::getId($row->pid));
			$wd->setShortId($row->pid);
			//$wd->setUri(WoogleMWUtil::getArticleAbsolutePath() . urlencode($ptitle));
			$wd->setUri(WoogleMWUtil::getArticleAbsolutePath() . $ptitle); // WOO-420
			$wd->setLabel($ptitle);
			$text = $row->txt;
			
			// TODO duplicated in articleToDocument
			if ( WoogleConfig::get('htmlIndexing') ){
				$text = WoogleMWUtil::parseArticleFromTitle($row->title, true); // TODO needs namespace?
				$text = strip_tags($text);
			} else {
				$text = WoogleMWUtil::stripMediaWikiMarkup($text);
			}
			$wd->setContent($text);
			$wd->setChanged($row->touched); // MediaWiki concatenated string timestamp (YYYYMMDDHHMMSS) 
			$wd->setCreated($row->touched); // TODO wrong

			if ($printStatus) echo ".";

			array_push($docs, $wd);
		} 


		// Add documents to index 

		if ($direct) {
			$this->doAddItems($docs, true, true, $printStatus);
		} else {
			$this->addItems($docs);
		}
		
		//
		// index file content
		//
		if ( WoogleConfig::get('indexFileContent') ) {
			
			// Find titles in database
			if ($printStatus) echo "Querying attached files from the database";
			$fDocs = $this->indexFileContent();
			
			// Add documents to index 
			if ($direct) {
				$this->doAddItems($fDocs, true, true, $printStatus);
			} else {
				$this->addItems($fDocs);
			}
			
		}

		// close and optimize index
		$endTimeCommit = time();
		if ($direct) $this->stopReIndex(); // only makes sense for direct indexing
		$endTimeOptimize = time();

		//
		// print information about the indexing process
		//
		
		$iTime = $endTimeCommit - $startTime;
		$oTime = $endTimeOptimize - $startTime;
		$wgOut->addHTML('end time commit: ' . date('c', $endTimeCommit) . '<br />');
		$wgOut->addHTML('end time optimize: ' . date('c', $endTimeOptimize) . '<br />');
		
		$wgOut->addHTML('runtime index import: ' . $iTime . ' seconds<br />');
		$wgOut->addHTML('runtime index import + optimize: ' . $oTime . ' seconds<br />');
		
		$iSize = -1;
		if ($direct){
			$iSize =  $this->getIndexFileSize(); 
			if ($iSize > 0){
				$wgOut->addHTML('index directory size: ' . $iSize . 'KB<br />');
				WoogleUtil::perflog($iSize . ';' . 'indexfilesize', 'indexall');
			}
		}
		
		WoogleUtil::debug("Indexing summary - " . 'end time optimize: ' . date('c', $endTimeOptimize));
		WoogleUtil::debug("Indexing summary - " . 'runtime index import + optimize: ' . $oTime . ' seconds');
		WoogleUtil::debug("Indexing summary - " . 'number of documents: ' . $this->numDocs() );
		WoogleUtil::perflog($direct . ';' . $this->numDocs() . ';' . $iSize . ';' . $iTime . ';' . $oTime, 'indexall');
		
		if ($this->numDocs() > 0){
			$wgOut->addHTML('number of documents: ' . $this->numDocs() . '<br />');
			$wgOut->addHTML('duration per document (using optimize time): ' . (($endTimeOptimize - $startTime) / $this->numDocs()) . '<br />');
		}
		
		if ($direct){
			$wgOut->addHTML('<b>Index created successfully!</b><br />');
		} else {
			$wgOut->addHTML('<b>Job queue was prepares successfully! The index will be iteratively.</b><br />');
		}
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * indexes content of uploaded files.
	 * 
	 * @return 			array of WoogleDocuments
	 */
	private function indexFileContent() {
		WoogleUtil::profileIn(__METHOD__);
		global $wgServer;

		$db = WoogleMWUtil::getDB();
		extract($db->tableNames('image'));

		$sql = 'SELECT img_name as name, img_major_mime as major, img_minor_mime as mime, img_size as size, img_timestamp as timestamp, ' .
		       'img_sha1 as sha1 FROM ' . $image;
		$res = $db->doQuery($sql);
		
		$fDocs = array();
		while ($row = $db->fetchObject($res)) {
			$filename = $row->name;

			// get local file path
			$ftitle = Title::makeTitleSafe( NS_FILE, $filename );
			$lfile = wfLocalFile( $ftitle );
			$localFilePath = $lfile->getPath();

			// get URI
			$uri = $lfile->getURL();
			$uri = $wgServer . $uri;

      		// get mime type
      		$major 	= $row->major;
			$mime 	= $row->mime;

     		// using the sha1 value as ID as there seems to be no other id for files
			$id = $row->sha1;

			if ( $lfile && $lfile->exists() && file_exists($localFilePath) ) {
	
				$wd = new WoogleDocument();
				$wd->setId(WoogleQueryServiceCommon::getId($id));
				$wd->setChanged($row->timestamp);
				$wd->setCreated($row->timestamp);
				$wd->setSize(intval($row->size));
				$wd->setUri($uri);
				$wd->setLabel($filename);
				$wd->setMimetype($major . '/' . $mime);
				
				// deal only with PDF for the moment
				if ( $mime == 'pdf' ) {
					$text = WooglePDFExtractor::pdfFile2string($localFilePath);
					if ($text == '') WoogleUtil::error('Could not extract content from PDF file: ' . $localFilePath);
					//WoogleUtil::debug('Text returned from PDFExtractor: ' . $text);
					
					//
					// create a WoogleDocument from PDF file
					//
					//$fauxPageID = $fauxPageID + 1;	// what is this good for?
					
					// temporary workaround: indexing the original text leads to a
					// corrupted lucene index (Luke warning: "read past EOF")
					//$text = iconv("UTF-8", "ISO-8859-1", $text);
					
					// temporary workaround: strip all non-alphanum chars
					//$text = preg_replace("/[^a-zA-Z0-9\s]/", "", $text);
					
					$wd->setContent($text);
					array_push($fDocs, $wd);
				
				} else if ( $major == 'image') {

					$text = '[[File:' . $filename . '|50px]]';
					$wd->setContent($text);
					array_push($fDocs, $wd);
					
				}

			} else {
				WoogleUtil::error('File ' . $localFilePath . ' can not be indexed.');
			}
		}
				
		WoogleUtil::profileOut(__METHOD__);
		return $fDocs;
	}

		
}
