<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

class WoogleRedLinks {
	private static $avgRecency;
	
	/**
	 * This method computes the HTML for the popup box with meta-information
	 * which is added to "red links" within the MediaWiki
	 * 
	 * @param $linkText			The Title of the RedLink. This String is URL-Decoded
	 * @param $pageTitle		The Title of the Page, on which the RedLink was found. This String is URL-Decoded	 
	 * @return 	array ( linkTitle,  // URL-Decoded 
	 * 					HtmlCode for tooltip, //URL-Decoded
	 * 					PrioLevel )
	 */
	public static function getRedLinkInfo($linkText, $pageTitle) {
				
		WoogleUtil::profileIn(__METHOD__);						
		WoogleUtil::loadMessages();		
				
		$days30 = time() - (60*60*24*30); // now - 30 days	

		/*
		$x = strlen($linkText);
		$p = strpos($linkText, " (page does not exist)");
		if ($p>0) $x = $p;
		$linkText = substr($linkText, 0, $x); // TODO
		*/
					
		WoogleUtil::debug( "STRPOS_talk for " . $linkText . " is " . strpos($linkText, " talk") . "x");
		//WoogleUtil::debug( "STRPOS_Woogle: for " . $linkText . " is " . strpos($linkText, "Woogle:"). "x");

		// TODO exclude Woogle and talk...
		WoogleUtil::debug( "STRPOS_Woogle: for " . $linkText . " is " . strpos($linkText, "Woogle:"). "x");
		if (strpos($linkText , "oogle:") == 1){  // TODO Woogle
			WoogleUtil::debug("1_Break for " . $linkText);
			continue;
		}
		if (strpos($linkText, " talk") > 0){
			WoogleUtil::debug("2_Break for " . $linkText);
			continue;
		}

		if ($linkText == null || $linkText == ''){
			WoogleUtil::debug('Could not create title object for text: ' . $linkText);
			continue;
		}

		$rIcon = '';	// recency icon
		$qIcon = '';	// query icon

		// page-related numbers
		$inlinks  = WoogleMWUtil::getNumInlinks($linkText);
		$watchers =	WoogleMWUtil::getNumWatches($linkText);
		
		$searchers 			= 0; // TODO remove (?)
		
		$queries 			= 0;
		$max_searchers 		= 0;
		$avglastdate		= 0;
		$direct_queries 	= 0;
		$direct_searchers	= 0;
		$direct_lastsdata	= 0;
		
		$i = 0;
		
		$relatedq = WoogleUtil::getQueryManager()->getRelatedQueries($linkText, 50, false);
		$num_relq = count($relatedq);
		
		WoogleUtil::debug('Found ' . $num_relq . ' related queries for ' . $linkText);
		
		$recentQueries 			= 0;
		$relatedQueries 		= '';
		$recentRelatedQueries 	= '';
		
		foreach($relatedq as $ineed){
			
			$max_searchers	= max($max_searchers, $ineed['sizeQueryUsers']);
			$queries		+= $ineed['needExecutions'];
			
			WoogleUtil::debug('Adding ' . $ineed['needExecutions'] . ' for query ' . $ineed['needString']);

			$relatedQueries .= $ineed['needString'] . ';'; // TODO not nice
			if ($days30 < $ineed['lastDate']){ 
				$recentQueries++;
				$recentRelatedQueries .= $ineed['needString'] . ';'; // TODO not nice
			}
			
			// check if it is the main query
			if ($ineed['needString'] == $linkText){
				$direct_searchers	= $ineed['sizeQueryUsers'];
				$direct_queries		= $ineed['needExecutions'];
				$direct_lastdate	= $ineed['lastDate'];	
			} else {
				$i++;	
				//$avgrecency			= ($i * $avgrecency + $ineed['recency'])/ ($i+1);
			}
		}

		//
		//	 	Assign priority icons
		//
		$qText = WoogleUtil::getMsg('RedLinksQueriedTimesByUsers', $queries, $max_searchers);
		$qText1S = WoogleUtil::getMsg('RedLinksQueriedTimesByOne', $queries );
		$qText11 = WoogleUtil::getMsg('RedLinksQueriedOnce' );

		$prio = 0;
		if ($watchers > 1 || $inlinks > 5 || $direct_queries > 3 || $max_searchers > 5 ){
			// priority 3
			$prio = 3;
			$img = WoogleUtil::getImagePath() . 'importance_1.gif';
			$qIcon = '<img src="' . $img . '" alt="'. $qText. '" title="' . htmlspecialchars($relatedQueries) . '" />';
			if (($queries > 1) && ($max_searchers > 1)){
				$qIcon .= '&nbsp;' . $qText;
			} else if (($queries > 1 ) && ($max_searchers == 1)){
				$qIcon .= '&nbsp;' . $qText1S;
			} else if ($queries == 1 ){
				$qIcon .= '&nbsp;' . $qText11;
			}
		} else if ($watchers > 0 || $inlinks > 2 || $queries > 5 || $max_searchers > 2){
			// priority 2
			$prio = 2;
			$img = WoogleUtil::getImagePath() . 'importance_2.gif';
			$qIcon = '<img src="' . $img . '" alt="'. $qText. '" title="' . htmlspecialchars($relatedQueries) . '" />';
			if (($queries > 1) && ($max_searchers > 1)){
				$qIcon .= '&nbsp;' . $qText;
			} else if (($queries > 1 ) && ($max_searchers == 1)){
				$qIcon .= '&nbsp;' . $qText1S;
			} else if ($queries == 1 ){
				$qIcon .= '&nbsp;' . $qText11;
			}
		} else {
			// priority 1
			$prio = 1;
			$img = WoogleUtil::getImagePath() . 'importance_3.gif';
			$qIcon = WoogleUtil::getMsg('RedLinksNoNeed');
		}
		
		if ($inlinks > 2) $qIcon .= '&nbsp;'.WoogleUtil::getMsg('RedLinksReferencedTimes', $inlinks);			
		
		//
		//		Add recency icon
		//		
		
		if ($recentQueries > 1){ // TODO check data
			$rText = WoogleUtil::getMsg('RedLinksRelatedSearches', $recentQueries);
			$rIcon = ' <img src="' . WoogleUtil::getImagePath() . 'recency_1.gif" alt="' . $rText . '" title="' . $recentRelatedQueries . '" />&nbsp;' . $rText;
		} else if ($recentQueries == 1){
			$rText = WoogleUtil::getMsg('RedLinksRelatedSearchOne');				
			$rIcon = ' <img src="' . WoogleUtil::getImagePath() . 'recency_1.gif" alt="' . $rText . '" title="' . $recentRelatedQueries . '" />&nbsp;' . $rText;
		}
		
		$table = '<table><tr><td>' . $qIcon . '<br />' . $rIcon . '</td></tr></table>';
		
		/*
		// Note:
		// Do NOT use this code snippet again without paying attention to the explicit usage 
		// of '?'. This could lead to problems (cf. http://octopus13.fzi.de:8080/browse/WOO-315)
		$table = '<table>
<tr><td rowspan="2" valign="top"><img src="' . self::getImportanceIcon($queries, $inlinks) . '" /></td>
<td><a href="' . WoogleMWUtil::getArticleAbsolutePath() . SpecialPage::getTitleFor('Whatlinkshere', $linkText)->getPrefixedText() . '">' . $inlinks . ' ' . WoogleUtil::getMsg('RedLinksInlinks') . '</a></td></tr>
<tr><td><a href="' . WoogleMWUtil::getArticleAbsolutePath() . WoogleUtil::getSpecialPageTitle('ResultInfo')->getPrefixedText() . '/' . $linkText . '">' . $queries . ' ' . WoogleUtil::getMsg('RedLinksQueries') . '</a></td></tr>
<tr><td rowspan="2" valign="top"><img src="' . self::getUserIcon($watchers, $searchers) . '" /></td>
<td>' . $watchers .  ' ' . WoogleUtil::getMsg('RedLinksWatchers') . ' (<a href="' . WoogleMWUtil::getArticleAbsolutePath() . $linkText . '?action=watch">'. WoogleUtil::getMsg('RedLinksAddWatch') .'</a>)</td></tr>
<tr><td>' . $searchers . ' ' . WoogleUtil::getMsg('RedLinksSearchers'). $itext . ' </td></tr>
</table>';
		*/
		
		//$table = "TEXT";
		
		//run hook for instrumentation addon
		$iLog = ';PGE:;' .$pageTitle. ';WAT:;' . $watchers . ';INL:;' . $inlinks . ';QUE:;' . $queries . ';RQL:;' . $num_relq . ';DQR:;' . $direct_queries . ';MXS:;' . $max_searchers . ';RCT:;' . $recentQueries;
	
		wfRunHooks('WoogleRedLinkIcons', array($linkText . ';' .  $iLog));

		$result = array($linkText , $table, $prio);					
		
//		WoogleUtil::info('Result: '.print_r($result, true));		
//		$attributes = array( 'Watches'=> $watchers,'Inlinks'=> $inlinks, 'Querys'=> $queries ,'Searchers'=> $max_searchers);
//		WoogleUtil::info('Attribute: '.print_r($attributes, true));
				
		WoogleUtil::profileOut(__METHOD__);
		return $result;
	}
	
	// currently not used in here or in WoogleSearch
	public static function getUserIcon($numWatchers, $numSearchers) {
		WoogleUtil::profileIn(__METHOD__);
		$sum = $numWatchers + $numSearchers;
		$icon;
		if ($sum <= 3) {
			$icon = '1';
		}
		elseif ($sum <= 9) {
			$icon = '2';
		}
		else {
			$icon = '3';
		}
		$img = WoogleUtil::getImagePath() . 'user_' . $icon . '.gif';
		WoogleUtil::profileOut(__METHOD__);
		return $img;
	}

	// currently not used in here or in WoogleSearch
	public static function getImportanceIcon($numQueries, $numInlinks = 0) {
		WoogleUtil::profileIn(__METHOD__);
		$sum = $numQueries + $numInlinks;
		$icon;
		if ($sum <= 3) {
			$icon = '5';
		}
		elseif ($sum <= 10) {
			$icon = '4';
		}
		elseif ($sum <= 25) {
			$icon = '3';
		}
		elseif ($sum <= 50) {
			$icon = '2';
		}
		else {
			$icon = '1';
		}
		$img = WoogleUtil::getImagePath() . 'importance_' . $icon . '.gif';
		WoogleUtil::profileOut(__METHOD__);
		return $img;
	}
	
	// never used in here or in WoogleSearch
	public static function getRecencyIcon($recency, $avgRecency) {
		WoogleUtil::profileIn(__METHOD__);
		$icon;
		if ($recency > (self::$avgRecency * 1.1)) {
			$icon = '1';
		}
		elseif ($recency > (self::$avgRecency * 0.9)) {
			$icon = '2';
		}
		else {
			$icon = '3';
		}
		$img = WoogleUtil::getImagePath() . 'recency_' . $icon . '.gif';
		WoogleUtil::profileOut(__METHOD__);
		return $img;
	}
	
	// never used in here or in WoogleSearch
	public static function getQualityIcon($pageRatio) {
		WoogleUtil::profileIn(__METHOD__);
		$icon;
		if (abs(1 - $pageRatio) <= 0.05) {
			$icon = '3';
		}
		else if (abs(1 - $pageRatio) <= 0.15) {
			$icon = '2';
		}
		else {
			$icon = '1';
		}
		$img = WoogleUtil::getImagePath() . 'quality_' . $icon . '.gif';
		WoogleUtil::profileOut(__METHOD__);
		return $img;
	}
	
	// never used in here or in WoogleSearch
	public static function getQualityText($pageRatio) {
		WoogleUtil::profileIn(__METHOD__);
		$text;
		if (abs(1 - $pageRatio) <= 0.05) {
			$text = WoogleUtil::getMsg('RedLinkQualityGreen');
		}
		else if (abs(1 - $pageRatio) <= 0.15) {
			$text = WoogleUtil::getMsg('RedLinkQualityYellow');
		}
		else {
			$text = WoogleUtil::getMsg('RedLinkQualityRed');
		}
		WoogleUtil::profileOut(__METHOD__);
		return $text;
	}
}
