<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

/**
 * MediaWiki job to store user clicks on search results (captured via redirect.php) in native or remote backend
 * 
 * TODO: due to malfunction, this job is currently not used (@see WoogleQueryService->addClick())
 * 
 */
class WoogleJobAddClick extends WoogleJob {
	
	/**
	 * Called for executing one addClick job entity (see addJob() method)
	 * @param $title		?
	 * @param $params		array with click logging information
	 * @param $id			?
	 * @return				void
	 */
	public function __construct($title, $params, $id = 0) {
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::debug('Constructing click job');
		parent::__construct('woogleAddClick', Title::newMainPage(), $params, $id);
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * Executes the job - i.e. takes the $params array and passes it to the queryService click logging method
	 * If that fails, the job is enqueued again
	 * 
	  * @return			boolean, if job was successful (job will deleted from queue if true) 
	 */
	protected function runJob() {
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::debug("Running click job for " . $this->params['url']);
		
		try {
			WoogleUtil::getQueryService()->doAddClick($this->params['searchString'], $this->params['url'], $this->params['position'], $this->params['session']);
		} catch (Exception $e) {
			// logging failed, enqueue again
			$eMsg = "Failed to run click log job for " . $this->params['searchString'] . " / url: ".  $this->params['url'] . " - " . $e;
			WoogleUtil::error($eMsg);
			$this->errorMsg = $eMsg;
			return WoogleUtil::profileOut(__METHOD__, false);
		}
		
		// success
		WoogleUtil::debug("Successfully run click log job for " . $this->params['searchString'] . " / url: ".  $this->params['url']);
		return WoogleUtil::profileOut(__METHOD__, true);
	}
	
	/**
	 * Add job for storing click
	 * @param $searchString		query for which result was clicked
	 * @param $url				result url
	 * @param $position			position of result in result list
	 * @param $session			session iD
	  * @return					void
	 */
	public static function addJob($searchString, $url, $position, $session) {
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::debug("Adding click log job for " .$searchString . " / url: " . $url);
		$job = new WoogleJobAddClick('', array('searchString' => $searchString, 'url' => $url, 'position' => $position, 'session' => $session));
		$job->insert();
		WoogleUtil::debug('Click job inserted');
		WoogleUtil::error('Click job inserted');
		WoogleUtil::profileOut(__METHOD__);
	}
}
