<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

class WoogleJobRefreshItem extends WoogleJob {
	
	/**
	 * Called for executing one refresh job (see addJob() method)
	 * @param $title		?
	 * @param $params		array with the article id
	 * @param $id			?
	 * @return				void
	 */
	public function __construct($title, $params, $id = 0) {
		WoogleUtil::profileIn(__METHOD__);
		parent::__construct('woogleRefreshItem', Title::newMainPage(), $params, $id);
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * Executes the job - i.e. takes the articleId and calls appropriate method in indexservice
	 * If that fails, the job is enqueued again
	 * 
	 * @return			boolean, if job was successful (job will deleted from queue if true) 
	 */
	protected function runJob() {
		WoogleUtil::profileIn(__METHOD__);
		try {
			WoogleUtil::debug('WoogleJobRefreshItem:runJob');
			
			$jStart = microtime(true);
			$jAction = 'undefined';
			
			$id 	= $this->params['articleId'];
			$title 	= Title::newFromID($id);
			$is 	= WoogleUtil::getIndexService();

			// omit excluded namespaces
			$namespaces = WoogleConfig::get('excludeNamespaces');
			if (in_array($title->getNamespace(), $namespaces)){
				WoogleUtil::debug('WoogleJobRefreshItem:skipping ' . $id);
				return WoogleUtil::profileOut(__METHOD__, true);
			}
			
			// do not index Media-NS article pages, if file content indexing is enabled
			if (WoogleConfig::get('indexFileContent') && ($title->getNamespace() == NS_FILE)){
				WoogleUtil::debug('WoogleJobRefreshItem:skipping ' . $id);
				return WoogleUtil::profileOut(__METHOD__, true);
			}
 			
			if ($title != null) {
				$article = new Article($title, 0); // 0 for current revision
				$a = $is->exists($id);
				WoogleUtil::debug('WoogleJobRefreshItem:exists worked');
				if ($a) {
					// update existing article
					$jAction = 'update';
					WoogleUtil::debug('WoogleJobRefreshItem:updating ' . WoogleQueryServiceCommon::getId($id) . '...');
					$is->doUpdateItem(WoogleUtil::articleToDocument($article));
				} else {
					// add new article
					$jAction = 'add';
					WoogleUtil::debug('WoogleJobRefreshItem:adding ' . WoogleQueryServiceCommon::getId($id) . '...');
					$is->doAddItem(WoogleUtil::articleToDocument($article));
				}
			} else {
				// delete article case
				$jAction = 'delete';
				WoogleUtil::debug('WoogleJobRefreshItem:deleting ' . WoogleQueryServiceCommon::getId($id) . '...');
				$is->doDeleteItem($id);
			}
			$jStop = microtime(true);
			$jTime = $jStop - $jStart;
			
			WoogleUtil::perflog($jTime . ';' . $jAction, 'indexupdate');
			WoogleUtil::debug('WoogleJobRefreshItem:successful!');
			
		} catch (Exception $e) {
			// adding failed, enqueue again
			$eMsg = 'WoogleJobRefreshItem:exception - ' . $e;
			WoogleUtil::error($eMsg);
			$this->errorMsg = $eMsg;
			return WoogleUtil::profileOut(__METHOD__, false);
		}
		return WoogleUtil::profileOut(__METHOD__, true);
	}
	
	/**
	 * Add job for refreshing an article
	 * @param $articleId		?, id of the article
	 * @return					void
	 */
	public static function addJob($articleId) {
		WoogleUtil::profileIn(__METHOD__);
		$job = new WoogleJobRefreshItem('', array('articleId' => intval($articleId)));
		$job->insert();
		WoogleUtil::profileOut(__METHOD__);
	}
}
