<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/  

/**
 * Allows to make a batch change of user preferences (see $replace array)
 * The script requires public read access to the Wiki currently (for API call)
 *
 * Note: if Woogle is not installed in its standard path under ./extensions
 *       then the MW_INSTALL_PATH environment variable must be set.
 *       See README in the maintenance directory.
 *
 * Usage: see printUsage() below
 *
 */

/*
 * no guarantees, but look in the usual place for commandLine.inc, so this
 * so it will work most of the time
 */
$optionsWithArgs = array( 'server' );
require_once ( getenv('MW_INSTALL_PATH') !== false
    ? getenv('MW_INSTALL_PATH')."/maintenance/commandLine.inc"
    : dirname( __FILE__ ) . '/../../../maintenance/commandLine.inc' );


	changeUserSettings();

function changeUserSettings(){
	
	//wait10seconds("\n  WARNING: Changing user preferences!\n");
	
	$replace = array(
		array('WoogleInstrumentationInstrumentation', 'WoogleInstInstEnabled'),
		array('WoogleInstrumentationInstrumentationGroup', 'WoogleInstInstGroup'),
		//array('WoogleInstrumentationInstrumenta', 'WoogleInstInstEnabled'),
	);
	
	// get all users via api (requires wiki to be readable)
	$params_array = array();	// stores the parameters for the API request
	$params_array['action']		= 'query';
	$params_array['list']		= 'allusers';
	$params_array['aulimit']	= 500;

	$params = new FauxRequest($params_array);
	
	$api = new ApiMain($params);
	$api->execute();
	$data = & $api->getResultData();
	
	echo 'Found ' . count($data['query']['allusers']) . ' users.' . "\n";
	
	// iterate over all users
	foreach ($data['query']['allusers'] as $usr) {
		$u = User::newFromName($usr['name']);
		$changed = false;
		
		// iterate prefs to replace
		foreach ($replace as $pair){
			$val = $u->getOption($pair[0]);
			if ($val != null){
				$u->setOption($pair[1], $val);
				echo 'Changing preference name ' . $pair[0] . ' to ' . $pair[1] . ' for user ' . $usr['name'] . "\n";
				$changed = true;
			} else {
				echo 'Nothing to change for preference name ' . $pair[0] . ' (value=null) ' . "\n";
			}
		}
	
		if ($changed) $u->saveSettings();
	}
	
	echo "\nDone.\n";

}

function wait10seconds($msg){
	
	global $options;
	if( isset( $options['f'] ) ) {
		// force execution, no waiting
		return;
	}
	
	echo $msg;
	$delay=10;
	echo "  Abort with CTRL-C in the next $delay seconds ...   ";
	
	for ($i = $delay+1; $i >= 1;) {
		echo str_repeat( chr(8), strlen( $i ) ) . str_repeat( chr(255), strlen( $i ) ) .
	    	 str_repeat( chr(8), strlen( $i ) ) . --$i;
		sleep(1);
	}
	
}

function printUsage(){
	echo '
 * USAGE INSTRUCTIONS
 * =======================================================
 * php WoogleChangeUserPreferences.php
 ';
}

?>