<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

/**
 * Rebuilds the Woogle search index. This is equivalent to clicking the respective
 * button on the page Special:WoogleConfig.
 *
 * Note: if Woogle is not installed in its standard path under ./extensions
 *       then the MW_INSTALL_PATH environment variable must be set.
 *       See README in the maintenance directory.
 *
 * Usage: see printUsage() below
 *
 */

/*
 * no guarantees, but look in the usual place for commandLine.inc, so this
 * so it will work most of the time
 */
$optionsWithArgs = array( 'server' );
require_once ( getenv('MW_INSTALL_PATH') !== false
    ? getenv('MW_INSTALL_PATH')."/maintenance/commandLine.inc"
    : dirname( __FILE__ ) . '/../../../maintenance/commandLine.inc' );

// check options
if( isset( $options['server'] ) ) {
	global $wgServer;
	$wgServer = $options['server'];
}

reIndex();

function reIndex(){
	
	global $wgServer;
	
	//
	// do error checks
	//
	if (stripos($wgServer, 'http://localhost') === 0) echo("\n  WARNING: \$wgServer begins with 'http://localhost'. URLs written to the index might not be accessible for users!\n");
	if (!WoogleUtil::getIndexService()->checkIndexService()) 	echo("\n  PROBLEM: Directory not writable or connection problem!\n");
	if (WoogleUtil::getQueryService()->isNative()){
		if (!WoogleLucene::getMbstringSupport()) 					echo("\n  PROBLEM: mb_string extension missing!\n");
		if (!WoogleLucene::getPcreSupport()) 						echo("\n  PROBLEM: PCRE extension missing!\n");
	}
	
	wait10seconds("\n  WARNING: Deleting existing index and creating new index!\n");
	
	$is = WoogleUtil::getIndexService();
	echo "\n  Using index at " . $is->getLocation() . "\n\n";
	$count = 0;
	$is->index(true, true);
	$count = $is->numDocs();
	
	while (ob_get_level() > 0) { // be sure to have some buffer, otherwise some PHPs complain
		ob_end_flush();
	}
	
	if ($count > -1) echo "\n  $count files have been added to the index.\n";
	
	echo "\n  Done.\n";
}

function wait10seconds($msg){
	
	global $options;
	if( isset( $options['f'] ) ) {
		// force execution, no waiting
		return;
	}
	
	echo $msg;
	$delay=10;
	echo "  Abort with CTRL-C in the next $delay seconds ...   ";
	
	for ($i = $delay+1; $i >= 1;) {
		echo str_repeat( chr(8), strlen( $i ) ) . str_repeat( chr(255), strlen( $i ) ) .
	    	 str_repeat( chr(8), strlen( $i ) ) . --$i;
		sleep(1);
	}
	
}

function printUsage(){
	echo '
 * USAGE INSTRUCTIONS
 * =======================================================
 * php WoogleReIndexAll.php [options...]
 *
 * --f			Force (Skip 10 second delay)
 * --server=...	Server name (e.g. http://mywiki.mydomain.de) - without trailing slashes
 *				Alternatively set $wgServer in LocalSettings.php
 *				(http://www.mediawiki.org/wiki/Manual:LocalSettings.php#Server_name). 
 ';
}

?>