<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

require_once('Woogle_TestCase.php');

class WoogleAlertManagerTest extends Woogle_TestCase{
	
	public function setUp(){
	    $this->prepareTestDb();
	}
	
	public function tearDown(){
	
	}
	
	/* Test methods */	
	
	public function testGetUserAlerts(){

		$w = array(
			'Main_Page'	=> array(1, 2, 3),
			'Woogle'	=> array(1),
			'Karlsruhe'	=> array(3),
		);
		
		// Case 1: title match
		
		$wd = new WoogleDocument();
		$wd->setId(WoogleQueryServiceCommon::getId('999'));
		$wd->setShortId('999');
		$wd->setLabel('Karlsruhe');
		$wd->setContent('This is content');
		$wd->setUri('http://localhost/wiki/Page999');
		
		$res = WoogleAlertManager::getUserAlerts( $w , $wd);
		
		// only one match
		$this->assertEquals(1, count($res));
		$this->assertEquals('Karlsruhe', $res[3][0]);
		
		// Case 2: content match
		
		$wd->setLabel('Test');
		$wd->setContent('The best tool is Woogle!');
		$res = WoogleAlertManager::getUserAlerts( $w , $wd);
		// only one match
		$this->assertEquals(1, count($res));
		$this->assertEquals('Woogle', $res[1][0]);
		
		// Case 3: Multiple users match
		
		$wd->setLabel('Main_Page');
		$wd->setContent('Test');
		$res = WoogleAlertManager::getUserAlerts( $w , $wd);
		// three users
		$this->assertEquals(3, count($res));
		$this->assertEquals('Main_Page', $res[1][0]);
		
		// Case 4: Multiple watches match

		$wd->setLabel('Main_Page');
		$wd->setContent('The best tool is Woogle!');
		$res = WoogleAlertManager::getUserAlerts( $w , $wd);
		// three users
		$this->assertEquals(3, count($res));
		$this->assertEquals('Main_Page', $res[2][0]);
		// two hits for one user
		$this->assertEquals(2, count($res[1]));

		
		// TODO extend?
		
		
	}
	
}



?>
