<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

require_once('Woogle_TestCase.php');

class WoogleMWUtilTest extends Woogle_TestCase{
	
	public function setUp(){
	    $this->prepareTestDb();
	}
	
	public function tearDown(){
		// TODO?
	}
	
	/* Test methods */
	
	public function testTableExists(){
		$this->assertTrue(WoogleMWUtil::tableExists('page'));
		//$this->assertTrue(WoogleMWUtil::tableExists('parsertest_page'));
		//$this->assertTrue(WoogleMWUtil::tableExists('parsertest_user'));
		$this->assertFalse(WoogleMWUtil::tableExists('boo'));
	}
	
	public function testIsEmailValid(){
		$this->assertFalse(WoogleMWUtil::isEmailValid('blubb'));
		$this->assertFalse(WoogleMWUtil::isEmailValid('blubb.com'));
		$this->assertTrue(WoogleMWUtil::isEmailValid('blubb@blubb.com'));
	}
	
    public function testCheckDbPrivileges(){
        $this->assertTrue(WoogleMWUtil::checkDbPrivileges('SELECT'));
		$this->assertTrue(WoogleMWUtil::checkDbPrivileges('DROP'));
    }
    
    public function testPaths(){
    	
    	$this->assertEquals(0, stripos(WoogleMWUtil::getAbsolutePath(), 'http'));
    	$this->assertEquals(0, stripos(WoogleMWUtil::getWikiAbsolutePath(), 'http'));
    	$this->assertTrue(strlen(WoogleMWUtil::getWikiPath()) > 0);
    	$this->assertEquals(0, stripos(WoogleMWUtil::getArticleAbsolutePath(), 'http'));
		$this->assertTrue(strlen(WoogleMWUtil::getArticlePath()) > 0);
    	    	
    }
	
	public function testGetNumWatches(){
		$this->assertEquals(1, WoogleMWUtil::getNumWatches('Main_Page'));
		$this->assertEquals(1, WoogleMWUtil::getNumWatches('Main Page'));
		$this->assertEquals(1, WoogleMWUtil::getNumWatches(' Main Page '));
		$this->assertEquals(1, WoogleMWUtil::getNumWatches('main_Page'));
		$this->assertEquals(0, WoogleMWUtil::getNumWatches('Test')); 				// defined for NS_MAIN
		$this->assertEquals(0, WoogleMWUtil::getNumWatches('test')); 				// defined for NS_MAIN
		$this->assertEquals(2, WoogleMWUtil::getNumWatches('Test', WOO_NS_MAIN)); 	// defined for NSWoogle
		$this->assertEquals(2, WoogleMWUtil::getNumWatches('test', WOO_NS_MAIN)); 	// defined for NSWoogle
		
		// TODO add results w. namespaces!
	}
	
	public function testGetNumInlinks(){
		$this->assertEquals(1, WoogleMWUtil::getNumInlinks('smithee'));
		$this->assertEquals(1, WoogleMWUtil::getNumInlinks('Smithee'));
		$this->assertEquals(1, WoogleMWUtil::getNumInlinks(' Smithee '));
		$this->assertEquals(1, WoogleMWUtil::getNumInlinks('Alan Smithee'));
		$this->assertEquals(1, WoogleMWUtil::getNumInlinks('alan Smithee'));
		$this->assertEquals(1, WoogleMWUtil::getNumInlinks('Alan_Smithee'));
		$this->assertEquals(1, WoogleMWUtil::getNumInlinks('Veränderung'));
		$this->assertEquals(1, WoogleMWUtil::getNumInlinks('veränderung'));
	}
	
	public function testGetWatches(){
		
		// get all watches
		$w = WoogleMWUtil::getWatches();
		$this->assertEquals(2, count($w));
		$this->assertEquals(2, count($w['Test']));
		$this->assertEquals(1, count($w['Main_Page']));
		$this->assertEquals(1, $w['Main_Page'][0]);		// userId
		
		// get only for one namespace
		$w = WoogleMWUtil::getWatches(WOO_NS_MAIN);
		$this->assertEquals(1, count($w));
		$this->assertEquals(2, count($w['Test']));
				
	}		

	public function testStripMediaWikiMarkup(){
		$input = "== Head ==\n This is my {{PAGE_NAME}} article! It is '''awesome''' - you __AGREE__? Große Späße mit Umlauten!";
		$stripped = WoogleMWUtil::stripMediaWikiMarkup($input);
		//echo "STRIPPED: " . $stripped;
		
		// assert removed stuff
		$this->assertFalse(strpos($stripped, 'PAGE'));
		$this->assertFalse(strpos($stripped, '{'));
		$this->assertFalse(strpos($stripped, '}'));
		$this->assertFalse(strpos($stripped, '='));
		$this->assertFalse(strpos($stripped, '\''));
		$this->assertFalse(strpos($stripped, 'AGREE'));
		
		// assert included stuff
		$this->assertTrue(strpos($stripped, 'This') > 0);
		$this->assertTrue(strpos($stripped, 'article') > 0);
		$this->assertTrue(strpos($stripped, 'awesome') > 0);
		$this->assertTrue(mb_strpos($stripped, 'Große Späße') > 0);
		
		// User defined markup
		//WoogleConfig::set('stripMWMarkup', array(array('REGEXP', '')));
		$partA = 'This is a test ';
		$badCode = '{{#ifloggedin: TEXT_FOR_LOGGED_IN_USERS | TEXT_FOR_USERS_NOT_LOGGED_IN }}';
		$partB = ' including markup to be strippped!';
		$input = $partA . $badCode . $partB;
		$stripped = WoogleMWUtil::stripMediaWikiMarkup($input);
		$this->assertEquals($partA . $partB, $stripped);
		
		// TODO add Table {CONTENT}
		
	}
	 
	public function testEndsWith(){
		$this->assertTrue(WoogleMWUtil::endsWith('file.pdf', '.pdf'));
		$this->assertFalse(WoogleMWUtil::endsWith('file.pdf.zip', '.pdf'));
		$this->assertFalse(WoogleMWUtil::endsWith('file.doc', '.pdf'));
	}

}


?>
